@@ -1,3 +1,4 @@
+'use strict';
 var common = require('../common.js');
 var events = require('events');
 
@@ -9,14 +10,15 @@ function main(conf) {
   var ee = new events.EventEmitter();
   var listeners = [];
 
-  for (var k = 0; k < 10; k += 1)
+  var k;
+  for (k = 0; k < 10; k += 1)
     listeners.push(function() {});
 
   bench.start();
   for (var i = 0; i < n; i += 1) {
-    for (var k = listeners.length; --k >= 0; /* empty */)
+    for (k = listeners.length; --k >= 0; /* empty */)
       ee.on('dummy', listeners[k]);
-    for (var k = listeners.length; --k >= 0; /* empty */)
+    for (k = listeners.length; --k >= 0; /* empty */)
       ee.removeListener('dummy', listeners[k]);
   }
   bench.end(n);
