@@ -21,7 +21,6 @@ const trailerExpression = /^Trailer$/i;
 const connectionExpression = /^Connection$/i;
 const connCloseExpression = /(^|\W)close(\W|$)/i;
 const connUpgradeExpression = /(^|\W)upgrade(\W|$)/i;
-const lenientHttpHeaders = !!process.REVERT_CVE_2016_2216;
 
 const automaticHeaders = {
   connection: true,
@@ -93,8 +92,10 @@ exports.OutgoingMessage = OutgoingMessage;
 
 
 OutgoingMessage.prototype.setTimeout = function(msecs, callback) {
-  if (callback)
+
+  if (callback) {
     this.on('timeout', callback);
+  }
 
   if (!this.socket) {
     this.once('socket', function(socket) {
@@ -127,12 +128,11 @@ OutgoingMessage.prototype._send = function(data, encoding, callback) {
   // this at a lower level and in a more general way.
   if (!this._headerSent) {
     if (typeof data === 'string' &&
-        encoding !== 'hex' &&
-        encoding !== 'base64') {
+        (encoding === 'utf8' || encoding === 'latin1' || !encoding)) {
       data = this._header + data;
     } else {
       this.output.unshift(this._header);
-      this.outputEncodings.unshift('binary');
+      this.outputEncodings.unshift('latin1');
       this.outputCallbacks.unshift(null);
       this.outputSize += this._header.length;
       if (typeof this._onPendingData === 'function')
@@ -309,14 +309,13 @@ OutgoingMessage.prototype._storeHeader = function(firstLine, headers) {
 };
 
 function storeHeader(self, state, field, value) {
-  if (!lenientHttpHeaders) {
-    if (!common._checkIsHttpToken(field)) {
-      throw new TypeError(
-        'Header name must be a valid HTTP Token ["' + field + '"]');
-    }
-    if (common._checkInvalidHeaderChar(value) === true) {
-      throw new TypeError('The header content contains invalid characters');
-    }
+  if (!common._checkIsHttpToken(field)) {
+    throw new TypeError(
+      'Header name must be a valid HTTP Token ["' + field + '"]');
+  }
+  if (common._checkInvalidHeaderChar(value) === true) {
+    debug('Header "%s" contains invalid characters', field);
+    throw new TypeError('The header content contains invalid characters');
   }
   state.messageHeader += field + ': ' + escapeHeaderValue(value) + CRLF;
 
@@ -348,20 +347,16 @@ function storeHeader(self, state, field, value) {
 
 
 OutgoingMessage.prototype.setHeader = function(name, value) {
-  if (typeof name !== 'string')
-    throw new TypeError('`name` should be a string in setHeader(name, value).');
+  if (!common._checkIsHttpToken(name))
+    throw new TypeError(
+      'Header name must be a valid HTTP Token ["' + name + '"]');
   if (value === undefined)
-    throw new Error('`value` required in setHeader("' + name + '", value).');
+    throw new Error('"value" required in setHeader("' + name + '", value)');
   if (this._header)
     throw new Error('Can\'t set headers after they are sent.');
-  if (!lenientHttpHeaders) {
-    if (!common._checkIsHttpToken(name)) {
-      throw new TypeError(
-        'Trailer name must be a valid HTTP Token ["' + name + '"]');
-    }
-    if (common._checkInvalidHeaderChar(value) === true) {
-      throw new TypeError('The header content contains invalid characters');
-    }
+  if (common._checkInvalidHeaderChar(value) === true) {
+    debug('Header "%s" contains invalid characters', name);
+    throw new TypeError('The header content contains invalid characters');
   }
   if (this._headers === null)
     this._headers = {};
@@ -377,7 +372,7 @@ OutgoingMessage.prototype.setHeader = function(name, value) {
 
 OutgoingMessage.prototype.getHeader = function(name) {
   if (arguments.length < 1) {
-    throw new Error('`name` is required for getHeader(name).');
+    throw new Error('"name" argument is required for getHeader(name)');
   }
 
   if (!this._headers) return;
@@ -389,11 +384,11 @@ OutgoingMessage.prototype.getHeader = function(name) {
 
 OutgoingMessage.prototype.removeHeader = function(name) {
   if (arguments.length < 1) {
-    throw new Error('`name` is required for removeHeader(name).');
+    throw new Error('"name" argument is required for removeHeader(name)');
   }
 
   if (this._header) {
-    throw new Error('Can\'t remove headers after they are sent.');
+    throw new Error('Can\'t remove headers after they are sent');
   }
 
   var key = name.toLowerCase();
@@ -412,7 +407,7 @@ OutgoingMessage.prototype.removeHeader = function(name) {
 
 OutgoingMessage.prototype._renderHeaders = function() {
   if (this._header) {
-    throw new Error('Can\'t render headers after they are sent to the client.');
+    throw new Error('Can\'t render headers after they are sent to the client');
   }
 
   var headersMap = this._headers;
@@ -429,6 +424,9 @@ OutgoingMessage.prototype._renderHeaders = function() {
   return headers;
 };
 
+OutgoingMessage.prototype._implicitHeader = function() {
+  throw new Error('_implicitHeader() method is not implemented');
+};
 
 Object.defineProperty(OutgoingMessage.prototype, 'headersSent', {
   configurable: true,
@@ -456,7 +454,7 @@ OutgoingMessage.prototype.write = function(chunk, encoding, callback) {
   }
 
   if (typeof chunk !== 'string' && !(chunk instanceof Buffer)) {
-    throw new TypeError('first argument must be a string or Buffer');
+    throw new TypeError('First argument must be a string or Buffer');
   }
 
 
@@ -469,7 +467,7 @@ OutgoingMessage.prototype.write = function(chunk, encoding, callback) {
     if (typeof chunk === 'string' &&
         encoding !== 'hex' &&
         encoding !== 'base64' &&
-        encoding !== 'binary') {
+        encoding !== 'latin1') {
       len = Buffer.byteLength(chunk, encoding);
       chunk = len.toString(16) + CRLF + chunk + CRLF;
       ret = this._send(chunk, encoding, callback);
@@ -484,7 +482,7 @@ OutgoingMessage.prototype.write = function(chunk, encoding, callback) {
         this.connection.cork();
         process.nextTick(connectionCorkNT, this.connection);
       }
-      this._send(len.toString(16), 'binary', null);
+      this._send(len.toString(16), 'latin1', null);
       this._send(crlf_buf, null, null);
       this._send(chunk, encoding, null);
       ret = this._send(crlf_buf, null, callback);
@@ -510,7 +508,6 @@ function connectionCorkNT(conn) {
 
 
 function escapeHeaderValue(value) {
-  if (!lenientHttpHeaders) return value;
   // Protect against response splitting. The regex test is there to
   // minimize the performance impact in the common case.
   return /[\r\n]/.test(value) ? value.replace(/[\r\n]+[ \t]*/g, '') : value;
@@ -531,21 +528,20 @@ OutgoingMessage.prototype.addTrailers = function(headers) {
       field = key;
       value = headers[key];
     }
-    if (!lenientHttpHeaders) {
-      if (!common._checkIsHttpToken(field)) {
-        throw new TypeError(
-          'Trailer name must be a valid HTTP Token ["' + field + '"]');
-      }
-      if (common._checkInvalidHeaderChar(value) === true) {
-        throw new TypeError('The header content contains invalid characters');
-      }
+    if (!common._checkIsHttpToken(field)) {
+      throw new TypeError(
+        'Trailer name must be a valid HTTP Token ["' + field + '"]');
+    }
+    if (common._checkInvalidHeaderChar(value) === true) {
+      debug('Trailer "%s" contains invalid characters', field);
+      throw new TypeError('The header content contains invalid characters');
     }
     this._trailer += field + ': ' + escapeHeaderValue(value) + CRLF;
   }
 };
 
 
-const crlf_buf = new Buffer('\r\n');
+const crlf_buf = Buffer.from('\r\n');
 
 
 OutgoingMessage.prototype.end = function(data, encoding, callback) {
@@ -558,21 +554,13 @@ OutgoingMessage.prototype.end = function(data, encoding, callback) {
   }
 
   if (data && typeof data !== 'string' && !(data instanceof Buffer)) {
-    throw new TypeError('first argument must be a string or Buffer');
+    throw new TypeError('First argument must be a string or Buffer');
   }
 
   if (this.finished) {
     return false;
   }
 
-  var self = this;
-  function finish() {
-    self.emit('finish');
-  }
-
-  if (typeof callback === 'function')
-    this.once('finish', callback);
-
   if (!this._header) {
     if (data) {
       if (typeof data === 'string')
@@ -600,11 +588,18 @@ OutgoingMessage.prototype.end = function(data, encoding, callback) {
     this.write(data, encoding);
   }
 
+  if (typeof callback === 'function')
+    this.once('finish', callback);
+
+  const finish = () => {
+    this.emit('finish');
+  };
+
   if (this._hasBody && this.chunkedEncoding) {
-    ret = this._send('0\r\n' + this._trailer + '\r\n', 'binary', finish);
+    ret = this._send('0\r\n' + this._trailer + '\r\n', 'latin1', finish);
   } else {
     // Force a flush, HACK.
-    ret = this._send('', 'binary', finish);
+    ret = this._send('', 'latin1', finish);
   }
 
   if (this.connection && data)
