@@ -1,5 +1,7 @@
 'use strict';
 
+require('internal/util').assertCrypto();
+
 const assert = require('assert');
 const EventEmitter = require('events');
 const stream = require('stream');
@@ -9,12 +11,7 @@ const common = require('_tls_common');
 const debug = util.debuglog('tls-legacy');
 const Buffer = require('buffer').Buffer;
 const Timer = process.binding('timer_wrap').Timer;
-var Connection = null;
-try {
-  Connection = process.binding('crypto').Connection;
-} catch (e) {
-  throw new Error('node.js not compiled with openssl crypto support.');
-}
+const Connection = process.binding('crypto').Connection;
 
 function SlabBuffer() {
   this.create();
@@ -23,7 +20,7 @@ function SlabBuffer() {
 
 SlabBuffer.prototype.create = function create() {
   this.isFull = false;
-  this.pool = new Buffer(tls.SLAB_BUFFER_SIZE);
+  this.pool = Buffer.allocUnsafe(tls.SLAB_BUFFER_SIZE);
   this.offset = 0;
   this.remaining = this.pool.length;
 };
@@ -353,14 +350,18 @@ CryptoStream.prototype.setKeepAlive = function(enable, initialDelay) {
   if (this.socket) this.socket.setKeepAlive(enable, initialDelay);
 };
 
-CryptoStream.prototype.__defineGetter__('bytesWritten', function() {
-  return this.socket ? this.socket.bytesWritten : 0;
+Object.defineProperty(CryptoStream.prototype, 'bytesWritten', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    return this.socket ? this.socket.bytesWritten : 0;
+  }
 });
 
 CryptoStream.prototype.getPeerCertificate = function(detailed) {
   if (this.pair.ssl) {
     return common.translatePeerCertificate(
-        this.pair.ssl.getPeerCertificate(detailed));
+      this.pair.ssl.getPeerCertificate(detailed));
   }
 
   return null;
@@ -424,9 +425,9 @@ CryptoStream.prototype.destroySoon = function(err) {
     // was written on this side was read from the other side.
     var self = this;
     var waiting = 1;
-    var finish = function() {
+    function finish() {
       if (--waiting === 0) self.destroy();
-    };
+    }
     this._opposite.once('end', finish);
     if (!this._finished) {
       this.once('finish', finish);
@@ -480,7 +481,7 @@ CryptoStream.prototype._done = function() {
 // readyState is deprecated. Don't use it.
 Object.defineProperty(CryptoStream.prototype, 'readyState', {
   get: function() {
-    if (this._connecting) {
+    if (this.connecting) {
       return 'opening';
     } else if (this.readable && this.writable) {
       return 'open';
@@ -529,27 +530,44 @@ CleartextStream.prototype.address = function() {
   return this.socket && this.socket.address();
 };
 
-
-CleartextStream.prototype.__defineGetter__('remoteAddress', function() {
-  return this.socket && this.socket.remoteAddress;
+Object.defineProperty(CleartextStream.prototype, 'remoteAddress', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    return this.socket && this.socket.remoteAddress;
+  }
 });
 
-CleartextStream.prototype.__defineGetter__('remoteFamily', function() {
-  return this.socket && this.socket.remoteFamily;
+Object.defineProperty(CleartextStream.prototype, 'remoteFamily', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    return this.socket && this.socket.remoteFamily;
+  }
 });
 
-CleartextStream.prototype.__defineGetter__('remotePort', function() {
-  return this.socket && this.socket.remotePort;
+Object.defineProperty(CleartextStream.prototype, 'remotePort', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    return this.socket && this.socket.remotePort;
+  }
 });
 
-
-CleartextStream.prototype.__defineGetter__('localAddress', function() {
-  return this.socket && this.socket.localAddress;
+Object.defineProperty(CleartextStream.prototype, 'localAddress', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    return this.socket && this.socket.localAddress;
+  }
 });
 
-
-CleartextStream.prototype.__defineGetter__('localPort', function() {
-  return this.socket && this.socket.localPort;
+Object.defineProperty(CleartextStream.prototype, 'localPort', {
+  configurable: true,
+  enumerable: true,
+  get: function() {
+    return this.socket && this.socket.localPort;
+  }
 });
 
 
@@ -590,7 +608,7 @@ function onhandshakestart() {
     // state machine and OpenSSL is not re-entrant. We cannot allow the user's
     // callback to destroy the connection right now, it would crash and burn.
     setImmediate(function() {
-      var err = new Error('TLS session renegotiation attack detected.');
+      var err = new Error('TLS session renegotiation attack detected');
       if (self.cleartext) self.cleartext.emit('error', err);
     });
   }
@@ -699,11 +717,12 @@ function SecurePair(context, isServer, requestCert, rejectUnauthorized,
   this._rejectUnauthorized = rejectUnauthorized ? true : false;
   this._requestCert = requestCert ? true : false;
 
-  this.ssl = new Connection(this.credentials.context,
-                            this._isServer ? true : false,
-                            this._isServer ? this._requestCert :
-                                             options.servername,
-                            this._rejectUnauthorized);
+  this.ssl = new Connection(
+    this.credentials.context,
+    this._isServer ? true : false,
+    this._isServer ? this._requestCert : options.servername,
+    this._rejectUnauthorized
+  );
 
   if (this._isServer) {
     this.ssl.onhandshakestart = () => onhandshakestart.call(this);
