@@ -4,16 +4,13 @@ const NativeModule = require('native_module');
 const util = require('util');
 const internalModule = require('internal/module');
 const internalUtil = require('internal/util');
-const runInThisContext = require('vm').runInThisContext;
+const vm = require('vm');
 const assert = require('assert').ok;
 const fs = require('fs');
 const path = require('path');
 const internalModuleReadFile = process.binding('fs').internalModuleReadFile;
 const internalModuleStat = process.binding('fs').internalModuleStat;
-
-const splitRe = process.platform === 'win32' ? /[\/\\]/ : /\//;
-const isIndexRe = /^index\.\w+?$/;
-const shebangRe = /^\#\!.*/;
+const preserveSymlinks = !!process.binding('config').preserveSymlinks;
 
 // If obj.hasOwnProperty has been overridden, then calling
 // obj.hasOwnProperty(prop) will break.
@@ -23,6 +20,20 @@ function hasOwnProperty(obj, prop) {
 }
 
 
+function stat(filename) {
+  filename = path._makeLong(filename);
+  const cache = stat.cache;
+  if (cache !== null) {
+    const result = cache.get(filename);
+    if (result !== undefined) return result;
+  }
+  const result = internalModuleStat(filename);
+  if (cache !== null) cache.set(filename, result);
+  return result;
+}
+stat.cache = null;
+
+
 function Module(id, parent) {
   this.id = id;
   this.exports = {};
@@ -70,8 +81,8 @@ function readPackage(requestPath) {
     return packageMainCache[requestPath];
   }
 
-  var jsonPath = path.resolve(requestPath, 'package.json');
-  var json = internalModuleReadFile(path._makeLong(jsonPath));
+  const jsonPath = path.resolve(requestPath, 'package.json');
+  const json = internalModuleReadFile(path._makeLong(jsonPath));
 
   if (json === undefined) {
     return false;
@@ -87,35 +98,47 @@ function readPackage(requestPath) {
   return pkg;
 }
 
-function tryPackage(requestPath, exts) {
+function tryPackage(requestPath, exts, isMain) {
   var pkg = readPackage(requestPath);
 
   if (!pkg) return false;
 
   var filename = path.resolve(requestPath, pkg);
-  return tryFile(filename) || tryExtensions(filename, exts) ||
-         tryExtensions(path.resolve(filename, 'index'), exts);
+  return tryFile(filename, isMain) ||
+         tryExtensions(filename, exts, isMain) ||
+         tryExtensions(path.resolve(filename, 'index'), exts, isMain);
 }
 
 // In order to minimize unnecessary lstat() calls,
 // this cache is a list of known-real paths.
-// Set to an empty object to reset.
-Module._realpathCache = {};
+// Set to an empty Map to reset.
+const realpathCache = new Map();
+
+const realpathCacheKey = fs.realpathCacheKey;
+delete fs.realpathCacheKey;
 
 // check if the file exists and is not a directory
-function tryFile(requestPath) {
-  const rc = internalModuleStat(path._makeLong(requestPath));
+// if using --preserve-symlinks and isMain is false,
+// keep symlinks intact, otherwise resolve to the
+// absolute realpath.
+function tryFile(requestPath, isMain) {
+  const rc = stat(requestPath);
+  if (preserveSymlinks && !isMain) {
+    return rc === 0 && path.resolve(requestPath);
+  }
   return rc === 0 && toRealPath(requestPath);
 }
 
 function toRealPath(requestPath) {
-  return fs.realpathSync(requestPath, Module._realpathCache);
+  return fs.realpathSync(requestPath, {
+    [realpathCacheKey]: realpathCache
+  });
 }
 
 // given a path check a the file exists with any of the set extensions
-function tryExtensions(p, exts) {
-  for (var i = 0, EL = exts.length; i < EL; i++) {
-    var filename = tryFile(p + exts[i]);
+function tryExtensions(p, exts, isMain) {
+  for (var i = 0; i < exts.length; i++) {
+    const filename = tryFile(p + exts[i], isMain);
 
     if (filename) {
       return filename;
@@ -125,48 +148,63 @@ function tryExtensions(p, exts) {
 }
 
 var warned = false;
-Module._findPath = function(request, paths) {
-  var exts = Object.keys(Module._extensions);
-
+Module._findPath = function(request, paths, isMain) {
   if (path.isAbsolute(request)) {
     paths = [''];
+  } else if (!paths || paths.length === 0) {
+    return false;
   }
 
-  var trailingSlash = (request.slice(-1) === '/');
-
-  var cacheKey = JSON.stringify({request: request, paths: paths});
+  const cacheKey = JSON.stringify({request: request, paths: paths});
   if (Module._pathCache[cacheKey]) {
     return Module._pathCache[cacheKey];
   }
 
+  var exts;
+  const trailingSlash = request.length > 0 &&
+                        request.charCodeAt(request.length - 1) === 47/*/*/;
+
   // For each path
-  for (var i = 0, PL = paths.length; i < PL; i++) {
+  for (var i = 0; i < paths.length; i++) {
     // Don't search further if path doesn't exist
-    if (paths[i] && internalModuleStat(path._makeLong(paths[i])) < 1) continue;
-    var basePath = path.resolve(paths[i], request);
+    const curPath = paths[i];
+    if (curPath && stat(curPath) < 1) continue;
+    var basePath = path.resolve(curPath, request);
     var filename;
 
     if (!trailingSlash) {
-      const rc = internalModuleStat(path._makeLong(basePath));
+      const rc = stat(basePath);
       if (rc === 0) {  // File.
-        filename = toRealPath(basePath);
+        if (preserveSymlinks && !isMain) {
+          filename = path.resolve(basePath);
+        } else {
+          filename = toRealPath(basePath);
+        }
       } else if (rc === 1) {  // Directory.
-        filename = tryPackage(basePath, exts);
+        if (exts === undefined)
+          exts = Object.keys(Module._extensions);
+        filename = tryPackage(basePath, exts, isMain);
       }
 
       if (!filename) {
         // try it with each of the extensions
-        filename = tryExtensions(basePath, exts);
+        if (exts === undefined)
+          exts = Object.keys(Module._extensions);
+        filename = tryExtensions(basePath, exts, isMain);
       }
     }
 
     if (!filename) {
-      filename = tryPackage(basePath, exts);
+      if (exts === undefined)
+        exts = Object.keys(Module._extensions);
+      filename = tryPackage(basePath, exts, isMain);
     }
 
     if (!filename) {
       // try it with each of the extensions at "index"
-      filename = tryExtensions(path.resolve(basePath, 'index'), exts);
+      if (exts === undefined)
+        exts = Object.keys(Module._extensions);
+      filename = tryExtensions(path.resolve(basePath, 'index'), exts, isMain);
     }
 
     if (filename) {
@@ -185,48 +223,120 @@ Module._findPath = function(request, paths) {
   return false;
 };
 
-// 'from' is the __dirname of the module.
-Module._nodeModulePaths = function(from) {
-  // guarantee that 'from' is absolute.
-  from = path.resolve(from);
+// 'node_modules' character codes reversed
+var nmChars = [ 115, 101, 108, 117, 100, 111, 109, 95, 101, 100, 111, 110 ];
+var nmLen = nmChars.length;
+if (process.platform === 'win32') {
+  // 'from' is the __dirname of the module.
+  Module._nodeModulePaths = function(from) {
+    // guarantee that 'from' is absolute.
+    from = path.resolve(from);
 
-  // note: this approach *only* works when the path is guaranteed
-  // to be absolute.  Doing a fully-edge-case-correct path.split
-  // that works on both Windows and Posix is non-trivial.
-  var paths = [];
-  var parts = from.split(splitRe);
+    // note: this approach *only* works when the path is guaranteed
+    // to be absolute.  Doing a fully-edge-case-correct path.split
+    // that works on both Windows and Posix is non-trivial.
 
-  for (var tip = parts.length - 1; tip >= 0; tip--) {
-    // don't search in .../node_modules/node_modules
-    if (parts[tip] === 'node_modules') continue;
-    var dir = parts.slice(0, tip + 1).concat('node_modules').join(path.sep);
-    paths.push(dir);
-  }
+    // return root node_modules when path is 'D:\\'.
+    // path.resolve will make sure from.length >=3 in Windows.
+    if (from.charCodeAt(from.length - 1) === 92/*\*/ &&
+        from.charCodeAt(from.length - 2) === 58/*:*/)
+      return [from + 'node_modules'];
 
-  return paths;
-};
+    const paths = [];
+    var p = 0;
+    var last = from.length;
+    for (var i = from.length - 1; i >= 0; --i) {
+      const code = from.charCodeAt(i);
+      // The path segment separator check ('\' and '/') was used to get
+      // node_modules path for every path segment.
+      // Use colon as an extra condition since we can get node_modules
+      // path for drive root like 'C:\node_modules' and don't need to
+      // parse drive name.
+      if (code === 92/*\*/ || code === 47/*/*/ || code === 58/*:*/) {
+        if (p !== nmLen)
+          paths.push(from.slice(0, last) + '\\node_modules');
+        last = i;
+        p = 0;
+      } else if (p !== -1) {
+        if (nmChars[p] === code) {
+          ++p;
+        } else {
+          p = -1;
+        }
+      }
+    }
+
+    return paths;
+  };
+} else { // posix
+  // 'from' is the __dirname of the module.
+  Module._nodeModulePaths = function(from) {
+    // guarantee that 'from' is absolute.
+    from = path.resolve(from);
+    // Return early not only to avoid unnecessary work, but to *avoid* returning
+    // an array of two items for a root: [ '//node_modules', '/node_modules' ]
+    if (from === '/')
+      return ['/node_modules'];
+
+    // note: this approach *only* works when the path is guaranteed
+    // to be absolute.  Doing a fully-edge-case-correct path.split
+    // that works on both Windows and Posix is non-trivial.
+    const paths = [];
+    var p = 0;
+    var last = from.length;
+    for (var i = from.length - 1; i >= 0; --i) {
+      const code = from.charCodeAt(i);
+      if (code === 47/*/*/) {
+        if (p !== nmLen)
+          paths.push(from.slice(0, last) + '/node_modules');
+        last = i;
+        p = 0;
+      } else if (p !== -1) {
+        if (nmChars[p] === code) {
+          ++p;
+        } else {
+          p = -1;
+        }
+      }
+    }
+
+    // Append /node_modules to handle root paths.
+    paths.push('/node_modules');
+
+    return paths;
+  };
+}
 
 
+// 'index.' character codes
+var indexChars = [ 105, 110, 100, 101, 120, 46 ];
+var indexLen = indexChars.length;
 Module._resolveLookupPaths = function(request, parent) {
   if (NativeModule.nonInternalExists(request)) {
     return [request, []];
   }
 
-  var start = request.substring(0, 2);
-  if (start !== './' && start !== '..') {
+  var reqLen = request.length;
+  // Check for relative path
+  if (reqLen < 2 ||
+      request.charCodeAt(0) !== 46/*.*/ ||
+      (request.charCodeAt(1) !== 46/*.*/ &&
+       request.charCodeAt(1) !== 47/*/*/)) {
     var paths = modulePaths;
     if (parent) {
-      if (!parent.paths) parent.paths = [];
-      paths = parent.paths.concat(paths);
+      if (!parent.paths)
+        paths = parent.paths = [];
+      else
+        paths = parent.paths.concat(paths);
     }
 
     // Maintain backwards compat with certain broken uses of require('.')
     // by putting the module's directory in front of the lookup paths.
     if (request === '.') {
       if (parent && parent.filename) {
-        paths.splice(0, 0, path.dirname(parent.filename));
+        paths.unshift(path.dirname(parent.filename));
       } else {
-        paths.splice(0, 0, path.resolve(request));
+        paths.unshift(path.resolve(request));
       }
     }
 
@@ -237,16 +347,46 @@ Module._resolveLookupPaths = function(request, parent) {
   if (!parent || !parent.id || !parent.filename) {
     // make require('./path/to/foo') work - normally the path is taken
     // from realpath(__filename) but with eval there is no filename
-    var mainPaths = ['.'].concat(modulePaths);
-    mainPaths = Module._nodeModulePaths('.').concat(mainPaths);
+    var mainPaths = ['.'].concat(Module._nodeModulePaths('.'), modulePaths);
     return [request, mainPaths];
   }
 
   // Is the parent an index module?
   // We can assume the parent has a valid extension,
   // as it already has been accepted as a module.
-  var isIndex = isIndexRe.test(path.basename(parent.filename));
-  var parentIdPath = isIndex ? parent.id : path.dirname(parent.id);
+  const base = path.basename(parent.filename);
+  var parentIdPath;
+  if (base.length > indexLen) {
+    var i = 0;
+    for (; i < indexLen; ++i) {
+      if (indexChars[i] !== base.charCodeAt(i))
+        break;
+    }
+    if (i === indexLen) {
+      // We matched 'index.', let's validate the rest
+      for (; i < base.length; ++i) {
+        const code = base.charCodeAt(i);
+        if (code !== 95/*_*/ &&
+            (code < 48/*0*/ || code > 57/*9*/) &&
+            (code < 65/*A*/ || code > 90/*Z*/) &&
+            (code < 97/*a*/ || code > 122/*z*/))
+          break;
+      }
+      if (i === base.length) {
+        // Is an index module
+        parentIdPath = parent.id;
+      } else {
+        // Not an index module
+        parentIdPath = path.dirname(parent.id);
+      }
+    } else {
+      // Not an index module
+      parentIdPath = path.dirname(parent.id);
+    }
+  } else {
+    // Not an index module
+    parentIdPath = path.dirname(parent.id);
+  }
   var id = path.resolve(parentIdPath, request);
 
   // make sure require('./path') and require('path') get distinct ids, even
@@ -274,7 +414,7 @@ Module._load = function(request, parent, isMain) {
     debug('Module._load REQUEST %s parent: %s', request, parent.id);
   }
 
-  var filename = Module._resolveFilename(request, parent);
+  var filename = Module._resolveFilename(request, parent, isMain);
 
   var cachedModule = Module._cache[filename];
   if (cachedModule) {
@@ -295,21 +435,24 @@ Module._load = function(request, parent, isMain) {
 
   Module._cache[filename] = module;
 
-  var hadException = true;
-
-  try {
-    module.load(filename);
-    hadException = false;
-  } finally {
-    if (hadException) {
-      delete Module._cache[filename];
-    }
-  }
+  tryModuleLoad(module, filename);
 
   return module.exports;
 };
 
-Module._resolveFilename = function(request, parent) {
+function tryModuleLoad(module, filename) {
+  var threw = true;
+  try {
+    module.load(filename);
+    threw = false;
+  } finally {
+    if (threw) {
+      delete Module._cache[filename];
+    }
+  }
+}
+
+Module._resolveFilename = function(request, parent, isMain) {
   if (NativeModule.nonInternalExists(request)) {
     return request;
   }
@@ -321,7 +464,7 @@ Module._resolveFilename = function(request, parent) {
   // look up the filename first, since that's the cache key.
   debug('looking for %j in %j', id, paths);
 
-  var filename = Module._findPath(request, paths);
+  var filename = Module._findPath(request, paths, isMain);
   if (!filename) {
     var err = new Error("Cannot find module '" + request + "'");
     err.code = 'MODULE_NOT_FOUND';
@@ -351,7 +494,7 @@ Module.prototype.load = function(filename) {
 Module.prototype.require = function(path) {
   assert(path, 'missing path');
   assert(typeof path === 'string', 'path must be a string');
-  return Module._load(path, this);
+  return Module._load(path, this, /* isMain */ false);
 };
 
 
@@ -365,15 +508,44 @@ var resolvedArgv;
 // the file.
 // Returns exception, if any.
 Module.prototype._compile = function(content, filename) {
-  // remove shebang
-  content = content.replace(shebangRe, '');
+  // Remove shebang
+  var contLen = content.length;
+  if (contLen >= 2) {
+    if (content.charCodeAt(0) === 35/*#*/ &&
+        content.charCodeAt(1) === 33/*!*/) {
+      if (contLen === 2) {
+        // Exact match
+        content = '';
+      } else {
+        // Find end of shebang line and slice it off
+        var i = 2;
+        for (; i < contLen; ++i) {
+          var code = content.charCodeAt(i);
+          if (code === 10/*\n*/ || code === 13/*\r*/)
+            break;
+        }
+        if (i === contLen)
+          content = '';
+        else {
+          // Note that this actually includes the newline character(s) in the
+          // new output. This duplicates the behavior of the regular expression
+          // that was previously used to replace the shebang line
+          content = content.slice(i);
+        }
+      }
+    }
+  }
 
   // create wrapper function
   var wrapper = Module.wrap(content);
 
-  var compiledWrapper = runInThisContext(wrapper,
-                                      { filename: filename, lineOffset: 0 });
-  if (global.v8debug) {
+  var compiledWrapper = vm.runInThisContext(wrapper, {
+    filename: filename,
+    lineOffset: 0,
+    displayErrors: true
+  });
+
+  if (process._debugWaitConnect && process._eval == null) {
     if (!resolvedArgv) {
       // we enter the repl if we're not given a filename argument.
       if (process.argv[1]) {
@@ -385,29 +557,19 @@ Module.prototype._compile = function(content, filename) {
 
     // Set breakpoint on module start
     if (filename === resolvedArgv) {
-      // Installing this dummy debug event listener tells V8 to start
-      // the debugger.  Without it, the setBreakPoint() fails with an
-      // 'illegal access' error.
-      global.v8debug.Debug.setListener(function() {});
-      global.v8debug.Debug.setBreakPoint(compiledWrapper, 0, 0);
+      delete process._debugWaitConnect;
+      const Debug = vm.runInDebugContext('Debug');
+      Debug.setBreakPoint(compiledWrapper, 0, 0);
     }
   }
-  const dirname = path.dirname(filename);
-  const require = internalModule.makeRequireFunction.call(this);
-
-  Object.defineProperty(require, 'paths', { get: function() {
-    throw new Error('require.paths is removed. Use ' +
-                    'node_modules folders, or the NODE_PATH ' +
-                    'environment variable instead.');
-  }});
-
-  require.registerExtension = function() {
-    throw new Error('require.registerExtension() removed. Use ' +
-                    'require.extensions instead.');
-  };
-
-  const args = [this.exports, require, this, filename, dirname];
-  return compiledWrapper.apply(this.exports, args);
+  var dirname = path.dirname(filename);
+  var require = internalModule.makeRequireFunction.call(this);
+  var args = [this.exports, require, this, filename, dirname];
+  var depth = internalModule.requireDepth;
+  if (depth === 0) stat.cache = new Map();
+  var result = compiledWrapper.apply(this.exports, args);
+  if (depth === 0) stat.cache = null;
+  return result;
 };
 
 
@@ -447,13 +609,23 @@ Module.runMain = function() {
 Module._initPaths = function() {
   const isWindows = process.platform === 'win32';
 
+  var homeDir;
   if (isWindows) {
-    var homeDir = process.env.USERPROFILE;
+    homeDir = process.env.USERPROFILE;
   } else {
-    var homeDir = process.env.HOME;
+    homeDir = process.env.HOME;
   }
 
-  var paths = [path.resolve(process.execPath, '..', '..', 'lib', 'node')];
+  // $PREFIX/lib/node, where $PREFIX is the root of the Node.js installation.
+  var prefixDir;
+  // process.execPath is $PREFIX/bin/node except on Windows where it is
+  // $PREFIX\node.exe.
+  if (isWindows) {
+    prefixDir = path.resolve(process.execPath, '..');
+  } else {
+    prefixDir = path.resolve(process.execPath, '..', '..');
+  }
+  var paths = [path.resolve(prefixDir, 'lib', 'node')];
 
   if (homeDir) {
     paths.unshift(path.resolve(homeDir, '.node_libraries'));
@@ -469,7 +641,7 @@ Module._initPaths = function() {
 
   modulePaths = paths;
 
-  // clone as a read-only copy, for introspection.
+  // clone as a shallow copy, for introspection.
   Module.globalPaths = modulePaths.slice(0);
 };
 
@@ -488,8 +660,7 @@ Module._preloadModules = function(requests) {
   var parent = new Module('internal/preload', null);
   try {
     parent.paths = Module._nodeModulePaths(process.cwd());
-  }
-  catch (e) {
+  } catch (e) {
     if (e.code !== 'ENOENT') {
       throw e;
     }
