@@ -24,6 +24,7 @@ var cluster;
 const errnoException = util._errnoException;
 const exceptionWithHostPort = util._exceptionWithHostPort;
 const isLegalPort = internalNet.isLegalPort;
+const assertPort = internalNet.assertPort;
 
 function noop() {}
 
@@ -65,6 +66,11 @@ exports.connect = exports.createConnection = function() {
   args = normalizeConnectArgs(args);
   debug('createConnection', args);
   var s = new Socket(args[0]);
+
+  if (args[0].timeout) {
+    s.setTimeout(args[0].timeout);
+  }
+
   return Socket.prototype.connect.apply(s, args);
 };
 
@@ -119,7 +125,7 @@ const BYTES_READ = Symbol('bytesRead');
 function Socket(options) {
   if (!(this instanceof Socket)) return new Socket(options);
 
-  this._connecting = false;
+  this.connecting = false;
   this._hadError = false;
   this._handle = null;
   this._parent = null;
@@ -199,13 +205,9 @@ Socket.prototype._unrefTimer = function _unrefTimer() {
 
 // the user has called .end(), and all the bytes have been
 // sent out to the other side.
-// If allowHalfOpen is false, or if the readable side has
-// ended already, then destroy.
-// If allowHalfOpen is true, then we need to do a shutdown,
-// so that only the writable side will be cleaned up.
 function onSocketFinish() {
   // If still connecting - defer handling 'finish' until 'connect' will happen
-  if (this._connecting) {
+  if (this.connecting) {
     debug('osF: not yet connected');
     return this.once('connect', onSocketFinish);
   }
@@ -370,9 +372,16 @@ Socket.prototype.address = function() {
 };
 
 
+Object.defineProperty(Socket.prototype, '_connecting', {
+  get: function() {
+    return this.connecting;
+  }
+});
+
+
 Object.defineProperty(Socket.prototype, 'readyState', {
   get: function() {
-    if (this._connecting) {
+    if (this.connecting) {
       return 'opening';
     } else if (this.readable && this.writable) {
       return 'open';
@@ -400,7 +409,7 @@ Object.defineProperty(Socket.prototype, 'bufferSize', {
 Socket.prototype._read = function(n) {
   debug('_read');
 
-  if (this._connecting || !this._handle) {
+  if (this.connecting || !this._handle) {
     debug('_read wait for connection');
     this.once('connect', () => this._read(n));
   } else if (!this._handle.reading) {
@@ -433,7 +442,7 @@ function maybeDestroy(socket) {
   if (!socket.readable &&
       !socket.writable &&
       !socket.destroyed &&
-      !socket._connecting &&
+      !socket.connecting &&
       !socket._writableState.length) {
     socket.destroy();
   }
@@ -454,9 +463,7 @@ Socket.prototype.destroySoon = function() {
 Socket.prototype._destroy = function(exception, cb) {
   debug('destroy');
 
-  var self = this;
-
-  function fireErrorCallbacks() {
+  function fireErrorCallbacks(self) {
     if (cb) cb(exception);
     if (exception && !self._writableState.errorEmitted) {
       process.nextTick(emitErrorNT, self, exception);
@@ -466,11 +473,11 @@ Socket.prototype._destroy = function(exception, cb) {
 
   if (this.destroyed) {
     debug('already destroyed, fire error callbacks');
-    fireErrorCallbacks();
+    fireErrorCallbacks(this);
     return;
   }
 
-  self._connecting = false;
+  this.connecting = false;
 
   this.readable = this.writable = false;
 
@@ -485,9 +492,9 @@ Socket.prototype._destroy = function(exception, cb) {
     // `bytesRead` should be accessible after `.destroy()`
     this[BYTES_READ] = this._handle.bytesRead;
 
-    this._handle.close(function() {
+    this._handle.close(() => {
       debug('emit close');
-      self.emit('close', isException);
+      this.emit('close', isException);
     });
     this._handle.onread = noop;
     this._handle = null;
@@ -498,7 +505,7 @@ Socket.prototype._destroy = function(exception, cb) {
   // to make it re-entrance safe in case Socket.prototype.destroy()
   // is called within callbacks
   this.destroyed = true;
-  fireErrorCallbacks();
+  fireErrorCallbacks(this);
 
   if (this._server) {
     COUNTER_NET_SERVER_CONNECTION_CLOSE(this);
@@ -643,8 +650,10 @@ protoGetter('localPort', function localPort() {
 
 
 Socket.prototype.write = function(chunk, encoding, cb) {
-  if (typeof chunk !== 'string' && !(chunk instanceof Buffer))
-    throw new TypeError('invalid data');
+  if (typeof chunk !== 'string' && !(chunk instanceof Buffer)) {
+    throw new TypeError(
+      'Invalid data, chunk must be a string or buffer, not ' + typeof chunk);
+  }
   return stream.Duplex.prototype.write.apply(this, arguments);
 };
 
@@ -653,7 +662,7 @@ Socket.prototype._writeGeneric = function(writev, data, encoding, cb) {
   // If we are still connecting, then buffer this for later.
   // The Writable logic will buffer up any more writes while
   // waiting for this one to be done.
-  if (this._connecting) {
+  if (this.connecting) {
     this._pendingData = data;
     this._pendingEncoding = encoding;
     this.once('connect', function() {
@@ -667,7 +676,7 @@ Socket.prototype._writeGeneric = function(writev, data, encoding, cb) {
   this._unrefTimer();
 
   if (!this._handle) {
-    this._destroy(new Error('This socket is closed.'), cb);
+    this._destroy(new Error('This socket is closed'), cb);
     return false;
   }
 
@@ -723,8 +732,9 @@ Socket.prototype._write = function(data, encoding, cb) {
 
 function createWriteReq(req, handle, data, encoding) {
   switch (encoding) {
+    case 'latin1':
     case 'binary':
-      return handle.writeBinaryString(req, data);
+      return handle.writeLatin1String(req, data);
 
     case 'buffer':
       return handle.writeBuffer(req, data);
@@ -743,7 +753,7 @@ function createWriteReq(req, handle, data, encoding) {
       return handle.writeUcs2String(req, data);
 
     default:
-      return handle.writeBuffer(req, new Buffer(data, encoding));
+      return handle.writeBuffer(req, Buffer.from(data, encoding));
   }
 }
 
@@ -807,7 +817,7 @@ function connect(self, address, port, addressType, localAddress, localPort) {
   // TODO return promise from Socket.prototype.connect which
   // wraps _connectReq.
 
-  assert.ok(self._connecting);
+  assert.ok(self.connecting);
 
   var err;
 
@@ -902,8 +912,9 @@ Socket.prototype.connect = function(options, cb) {
     this._sockname = null;
   }
 
-  var pipe = !!options.path;
-  debug('pipe', pipe, options.path);
+  const path = options.path;
+  var pipe = !!path;
+  debug('pipe', pipe, path);
 
   if (!this._handle) {
     this._handle = pipe ? new Pipe() : new TCP();
@@ -916,11 +927,14 @@ Socket.prototype.connect = function(options, cb) {
 
   this._unrefTimer();
 
-  this._connecting = true;
+  this.connecting = true;
   this.writable = true;
 
   if (pipe) {
-    connect(this, options.path);
+    if (typeof path !== 'string') {
+      throw new TypeError('"path" option must be a string: ' + path);
+    }
+    connect(this, path);
   } else {
     lookupAndConnect(this, options);
   }
@@ -936,16 +950,18 @@ function lookupAndConnect(self, options) {
   var localPort = options.localPort;
 
   if (localAddress && !exports.isIP(localAddress))
-    throw new TypeError('localAddress must be a valid IP: ' + localAddress);
+    throw new TypeError('"localAddress" option must be a valid IP: ' +
+                        localAddress);
 
   if (localPort && typeof localPort !== 'number')
-    throw new TypeError('localPort should be a number: ' + localPort);
+    throw new TypeError('"localPort" option should be a number: ' + localPort);
 
   if (typeof port !== 'undefined') {
     if (typeof port !== 'number' && typeof port !== 'string')
-      throw new TypeError('port should be a number or string: ' + port);
+      throw new TypeError('"port" option should be a number or string: ' +
+                          port);
     if (!isLegalPort(port))
-      throw new RangeError('port should be >= 0 and < 65536: ' + port);
+      throw new RangeError('"port" option should be >= 0 and < 65536: ' + port);
   }
   port |= 0;
 
@@ -953,29 +969,22 @@ function lookupAndConnect(self, options) {
   var addressType = exports.isIP(host);
   if (addressType) {
     process.nextTick(function() {
-      if (self._connecting)
+      if (self.connecting)
         connect(self, host, port, addressType, localAddress, localPort);
     });
     return;
   }
 
   if (options.lookup && typeof options.lookup !== 'function')
-    throw new TypeError('options.lookup should be a function.');
+    throw new TypeError('"lookup" option should be a function');
 
   var dnsopts = {
     family: options.family,
-    hints: 0
+    hints: options.hints || 0
   };
 
-  if (dnsopts.family !== 4 && dnsopts.family !== 6) {
+  if (dnsopts.family !== 4 && dnsopts.family !== 6 && dnsopts.hints === 0) {
     dnsopts.hints = dns.ADDRCONFIG;
-    // The AI_V4MAPPED hint is not supported on FreeBSD or Android,
-    // and getaddrinfo returns EAI_BADFLAGS. However, it seems to be
-    // supported on most other systems. See
-    // http://lists.freebsd.org/pipermail/freebsd-bugs/2008-February/028260.html
-    // for more information on the lack of support for FreeBSD.
-    if (process.platform !== 'freebsd' && process.platform !== 'android')
-      dnsopts.hints |= dns.V4MAPPED;
   }
 
   debug('connect: find host ' + host);
@@ -983,12 +992,12 @@ function lookupAndConnect(self, options) {
   self._host = host;
   var lookup = options.lookup || dns.lookup;
   lookup(host, dnsopts, function(err, ip, addressType) {
-    self.emit('lookup', err, ip, addressType);
+    self.emit('lookup', err, ip, addressType, host);
 
     // It's possible we were destroyed while looking this up.
     // XXX it would be great if we could cancel the promise returned by
     // the look up.
-    if (!self._connecting) return;
+    if (!self.connecting) return;
 
     if (err) {
       // net.createConnection() creates a net.Socket object and
@@ -1056,8 +1065,8 @@ function afterConnect(status, handle, req, readable, writable) {
 
   debug('afterConnect');
 
-  assert.ok(self._connecting);
-  self._connecting = false;
+  assert.ok(self.connecting);
+  self.connecting = false;
   self._sockname = null;
 
   if (status === 0) {
@@ -1073,7 +1082,7 @@ function afterConnect(status, handle, req, readable, writable) {
       self.read(0);
 
   } else {
-    self._connecting = false;
+    self.connecting = false;
     var details;
     if (req.localAddress && req.localPort) {
       details = req.localAddress + ':' + req.localPort;
@@ -1098,33 +1107,33 @@ function Server(options, connectionListener) {
 
   EventEmitter.call(this);
 
-  var self = this;
-
   if (typeof options === 'function') {
     connectionListener = options;
     options = {};
-    self.on('connection', connectionListener);
-  } else {
+    this.on('connection', connectionListener);
+  } else if (options == null || typeof options === 'object') {
     options = options || {};
 
     if (typeof connectionListener === 'function') {
-      self.on('connection', connectionListener);
+      this.on('connection', connectionListener);
     }
+  } else {
+    throw new TypeError('options must be an object');
   }
 
   this._connections = 0;
 
   Object.defineProperty(this, 'connections', {
-    get: internalUtil.deprecate(function() {
+    get: internalUtil.deprecate(() => {
 
-      if (self._usingSlaves) {
+      if (this._usingSlaves) {
         return null;
       }
-      return self._connections;
+      return this._connections;
     }, 'Server.connections property is deprecated. ' +
        'Use Server.getConnections method instead.'),
-    set: internalUtil.deprecate(function(val) {
-      return (self._connections = val);
+    set: internalUtil.deprecate((val) => {
+      return (this._connections = val);
     }, 'Server.connections property is deprecated.'),
     configurable: true, enumerable: false
   });
@@ -1333,7 +1342,7 @@ Server.prototype.listen = function() {
     self.once('listening', lastArg);
   }
 
-  var port = toNumber(arguments[0]);
+  var port = typeof arguments[0] === 'undefined' ? 0 : toNumber(arguments[0]);
 
   // The third optional argument is the backlog size.
   // When the ip is omitted it can be the second argument.
@@ -1360,8 +1369,7 @@ Server.prototype.listen = function() {
           (typeof h.port === 'undefined' && 'port' in h)) {
         // Undefined is interpreted as zero (random port) for consistency
         // with net.connect().
-        if (typeof h.port !== 'undefined' && !isLegalPort(h.port))
-          throw new RangeError('port should be >= 0 and < 65536: ' + h.port);
+        assertPort(h.port);
         if (h.host)
           listenAfterLookup(h.port | 0, h.host, backlog, h.exclusive);
         else
@@ -1382,10 +1390,12 @@ Server.prototype.listen = function() {
              typeof arguments[1] === 'function' ||
              typeof arguments[1] === 'number') {
     // The first argument is the port, no IP given.
+    assertPort(port);
     listen(self, null, port, 4, backlog);
 
   } else {
     // The first argument is the port, the second an IP.
+    assertPort(port);
     listenAfterLookup(port, arguments[1], backlog);
   }
 
@@ -1403,6 +1413,14 @@ Server.prototype.listen = function() {
   return self;
 };
 
+Object.defineProperty(Server.prototype, 'listening', {
+  get: function() {
+    return !!this._handle;
+  },
+  configurable: true,
+  enumerable: true
+});
+
 Server.prototype.address = function() {
   if (this._handle && this._handle.getsockname) {
     var out = {};
@@ -1481,13 +1499,6 @@ Server.prototype.getConnections = function(cb) {
 
 
 Server.prototype.close = function(cb) {
-  function onSlaveClose() {
-    if (--left !== 0) return;
-
-    self._connections = 0;
-    self._emitCloseIfDrained();
-  }
-
   if (typeof cb === 'function') {
     if (!this._handle) {
       this.once('close', function() {
@@ -1504,8 +1515,13 @@ Server.prototype.close = function(cb) {
   }
 
   if (this._usingSlaves) {
-    var self = this;
     var left = this._slaves.length;
+    const onSlaveClose = () => {
+      if (--left !== 0) return;
+
+      this._connections = 0;
+      this._emitCloseIfDrained();
+    };
 
     // Increment connections to be sure that, even if all sockets will be closed
     // during polling of slaves, `close` event will be emitted only once.
