@@ -24,12 +24,13 @@
 const internalModule = require('internal/module');
 const internalUtil = require('internal/util');
 const util = require('util');
+const utilBinding = process.binding('util');
 const inherits = util.inherits;
 const Stream = require('stream');
 const vm = require('vm');
 const path = require('path');
 const fs = require('fs');
-const rl = require('readline');
+const Interface = require('readline').Interface;
 const Console = require('console').Console;
 const Module = require('module');
 const domain = require('domain');
@@ -38,13 +39,13 @@ const debug = util.debuglog('repl');
 const parentModule = module;
 const replMap = new WeakMap();
 
-const GLOBAL_OBJECT_PROPERTIES = ['NaN', 'Infinity', 'undefined',
-  'eval', 'parseInt', 'parseFloat', 'isNaN', 'isFinite', 'decodeURI',
-  'decodeURIComponent', 'encodeURI', 'encodeURIComponent',
-  'Object', 'Function', 'Array', 'String', 'Boolean', 'Number',
-  'Date', 'RegExp', 'Error', 'EvalError', 'RangeError',
-  'ReferenceError', 'SyntaxError', 'TypeError', 'URIError',
-  'Math', 'JSON'];
+const GLOBAL_OBJECT_PROPERTIES = [
+  'NaN', 'Infinity', 'undefined', 'eval', 'parseInt', 'parseFloat', 'isNaN',
+  'isFinite', 'decodeURI', 'decodeURIComponent', 'encodeURI',
+  'encodeURIComponent', 'Object', 'Function', 'Array', 'String', 'Boolean',
+  'Number', 'Date', 'RegExp', 'Error', 'EvalError', 'RangeError',
+  'ReferenceError', 'SyntaxError', 'TypeError', 'URIError', 'Math', 'JSON'
+];
 const GLOBAL_OBJECT_PROPERTY_MAP = {};
 GLOBAL_OBJECT_PROPERTIES.forEach((p) => GLOBAL_OBJECT_PROPERTY_MAP[p] = p);
 
@@ -60,7 +61,7 @@ try {
 }
 
 // hack for repl require to work properly with node_modules folders
-module.paths = require('module')._nodeModulePaths(module.filename);
+module.paths = Module._nodeModulePaths(module.filename);
 
 // If obj.hasOwnProperty has been overridden, then calling
 // obj.hasOwnProperty(prop) will break.
@@ -74,10 +75,7 @@ function hasOwnProperty(obj, prop) {
 // This is the default "writer" value if none is passed in the REPL options.
 exports.writer = util.inspect;
 
-exports._builtinLibs = ['assert', 'buffer', 'child_process', 'cluster',
-  'crypto', 'dgram', 'dns', 'domain', 'events', 'fs', 'http', 'https', 'net',
-  'os', 'path', 'punycode', 'querystring', 'readline', 'stream',
-  'string_decoder', 'tls', 'tty', 'url', 'util', 'v8', 'vm', 'zlib'];
+exports._builtinLibs = internalModule.builtinLibs;
 
 
 const BLOCK_SCOPED_ERROR = 'Block-scoped declarations (let, const, function, ' +
@@ -95,6 +93,7 @@ class LineParser {
     this.shouldFail = false;
     this.blockComment = false;
     this.regExpLiteral = false;
+    this.prevTokenChar = null;
   }
 
   parseLine(line) {
@@ -134,7 +133,11 @@ class LineParser {
         if (previous === '/') {
           if (current === '*') {
             this.blockComment = true;
-          } else {
+          } else if (
+            // Distinguish between a division operator and the start of a regex
+            // by examining the non-whitespace character that precedes the /
+            [null, '(', '[', '{', '}', ';'].includes(this.prevTokenChar)
+          ) {
             this.regExpLiteral = true;
           }
           previous = null;
@@ -149,6 +152,8 @@ class LineParser {
         this._literal = this._literal || current;
       }
 
+      if (current.trim() && current !== '/') this.prevTokenChar = current;
+
       previous = current;
     }
 
@@ -169,7 +174,7 @@ class LineParser {
 
     this.shouldFail = this.shouldFail ||
                       ((!this._literal && lastChar === '\\') ||
-                      (this._literal   && lastChar !== '\\'));
+                      (this._literal && lastChar !== '\\'));
 
     return line;
   }
@@ -191,7 +196,7 @@ function REPLServer(prompt,
                           replMode);
   }
 
-  var options, input, output, dom;
+  var options, input, output, dom, breakEvalOnSigint;
   if (prompt !== null && typeof prompt === 'object') {
     // an options object was given
     options = prompt;
@@ -204,12 +209,17 @@ function REPLServer(prompt,
     prompt = options.prompt;
     dom = options.domain;
     replMode = options.replMode;
-  } else if (typeof prompt !== 'string') {
-    throw new Error('An options Object, or a prompt String are required');
+    breakEvalOnSigint = options.breakEvalOnSigint;
   } else {
     options = {};
   }
 
+  if (breakEvalOnSigint && eval_) {
+    // Allowing this would not reflect user expectations.
+    // breakEvalOnSigint affects only the behavior of the default eval().
+    throw new Error('Cannot specify both breakEvalOnSigint and eval for REPL');
+  }
+
   var self = this;
 
   self._domain = dom || domain.create();
@@ -217,6 +227,10 @@ function REPLServer(prompt,
   self.useGlobal = !!useGlobal;
   self.ignoreUndefined = !!ignoreUndefined;
   self.replMode = replMode || exports.REPL_MODE_SLOPPY;
+  self.underscoreAssigned = false;
+  self.last = undefined;
+  self.breakEvalOnSigint = !!breakEvalOnSigint;
+  self.editorMode = false;
 
   self._inTemplateLiteral = false;
 
@@ -231,9 +245,39 @@ function REPLServer(prompt,
 
   eval_ = eval_ || defaultEval;
 
+  function preprocess(code) {
+    let cmd = code;
+    if (/^\s*\{/.test(cmd) && /\}\s*$/.test(cmd)) {
+      // It's confusing for `{ a : 1 }` to be interpreted as a block
+      // statement rather than an object literal.  So, we first try
+      // to wrap it in parentheses, so that it will be interpreted as
+      // an expression.
+      cmd = `(${cmd})`;
+      self.wrappedCmd = true;
+    } else {
+      // Mitigate https://github.com/nodejs/node/issues/548
+      cmd = cmd.replace(
+        /^\s*function(?:\s*(\*)\s*|\s+)([^(]+)/,
+        (_, genStar, name) => `var ${name} = function ${genStar || ''}${name}`
+      );
+    }
+    // Append a \n so that it will be either
+    // terminated, or continued onto the next expression if it's an
+    // unexpected end of input.
+    return `${cmd}\n`;
+  }
+
   function defaultEval(code, context, file, cb) {
-    var err, result, retry = false;
+    // Remove trailing new line
+    code = code.replace(/\n$/, '');
+    code = preprocess(code);
+
+    var err, result, retry = false, input = code, wrappedErr;
     // first, create the Script object to check the syntax
+
+    if (code === '\n')
+      return cb(null);
+
     while (true) {
       try {
         if (!/^\s*$/.test(code) &&
@@ -244,20 +288,29 @@ function REPLServer(prompt,
         }
         var script = vm.createScript(code, {
           filename: file,
-          displayErrors: false
+          displayErrors: true
         });
       } catch (e) {
         debug('parse error %j', code, e);
         if (self.replMode === exports.REPL_MODE_MAGIC &&
             e.message === BLOCK_SCOPED_ERROR &&
-            !retry) {
-          retry = true;
+            !retry || self.wrappedCmd) {
+          if (self.wrappedCmd) {
+            self.wrappedCmd = false;
+            // unwrap and try again
+            code = `${input.substring(1, input.length - 2)}\n`;
+            wrappedErr = e;
+          } else {
+            retry = true;
+          }
           continue;
         }
-        if (isRecoverableError(e, self))
-          err = new Recoverable(e);
+        // preserve original error for wrapped command
+        const error = wrappedErr || e;
+        if (isRecoverableError(error, self))
+          err = new Recoverable(error);
         else
-          err = e;
+          err = error;
       }
       break;
     }
@@ -267,14 +320,46 @@ function REPLServer(prompt,
     regExMatcher.test(savedRegExMatches.join(sep));
 
     if (!err) {
+      // Unset raw mode during evaluation so that Ctrl+C raises a signal.
+      let previouslyInRawMode;
+      if (self.breakEvalOnSigint) {
+        // Start the SIGINT watchdog before entering raw mode so that a very
+        // quick Ctrl+C doesn’t lead to aborting the process completely.
+        utilBinding.startSigintWatchdog();
+        previouslyInRawMode = self._setRawMode(false);
+      }
+
       try {
-        if (self.useGlobal) {
-          result = script.runInThisContext({ displayErrors: false });
-        } else {
-          result = script.runInContext(context, { displayErrors: false });
+        try {
+          const scriptOptions = {
+            displayErrors: false,
+            breakOnSigint: self.breakEvalOnSigint
+          };
+
+          if (self.useGlobal) {
+            result = script.runInThisContext(scriptOptions);
+          } else {
+            result = script.runInContext(context, scriptOptions);
+          }
+        } finally {
+          if (self.breakEvalOnSigint) {
+            // Reset terminal mode to its previous value.
+            self._setRawMode(previouslyInRawMode);
+
+            // Returns true if there were pending SIGINTs *after* the script
+            // has terminated without being interrupted itself.
+            if (utilBinding.stopSigintWatchdog()) {
+              self.emit('SIGINT');
+            }
+          }
         }
       } catch (e) {
         err = e;
+        if (err.message === 'Script execution interrupted.') {
+          // The stack trace for this case is not very useful anyway.
+          Object.defineProperty(err, 'stack', { value: '' });
+        }
+
         if (err && process.domain) {
           debug('not recoverable, send to domain');
           process.domain.emit('error', err);
@@ -299,7 +384,12 @@ function REPLServer(prompt,
     debug('domain error');
     const top = replMap.get(self);
     internalUtil.decorateErrorStack(e);
-    if (e.stack && self.replMode === exports.REPL_MODE_STRICT) {
+    if (e instanceof SyntaxError && e.stack) {
+      // remove repl:line-number and stack trace
+      e.stack = e.stack
+                 .replace(/^repl:\d+\r?\n/, '')
+                 .replace(/^\s+at\s.*\n?/gm, '');
+    } else if (e.stack && self.replMode === exports.REPL_MODE_STRICT) {
       e.stack = e.stack.replace(/(\s+at\s+repl:)(\d+)/,
                                 (_, pre, line) => pre + (line - 1));
     }
@@ -335,20 +425,25 @@ function REPLServer(prompt,
   self.bufferedCommand = '';
   self.lines.level = [];
 
-  function complete(text, callback) {
-    self.complete(text, callback);
+  // Figure out which "complete" function to use.
+  self.completer = (typeof options.completer === 'function')
+    ? options.completer
+    : completer;
+
+  function completer(text, cb) {
+    complete.call(self, text, self.editorMode
+      ? self.completeOnEditorMode(cb) : cb);
   }
 
-  rl.Interface.call(this, {
+  Interface.call(this, {
     input: self.inputStream,
     output: self.outputStream,
-    completer: complete,
+    completer: self.completer,
     terminal: options.terminal,
-    historySize: options.historySize
+    historySize: options.historySize,
+    prompt
   });
 
-  self.setPrompt(prompt !== undefined ? prompt : '> ');
-
   this.commands = Object.create(null);
   defineDefaultCommands(this);
 
@@ -367,16 +462,16 @@ function REPLServer(prompt,
     };
   }
 
-  self.setPrompt(self._prompt);
-
   self.on('close', function() {
     self.emit('exit');
   });
 
   var sawSIGINT = false;
+  var sawCtrlD = false;
   self.on('SIGINT', function() {
     var empty = self.line.length === 0;
     self.clearLine();
+    self.turnOffEditorMode();
 
     if (!(self.bufferedCommand && self.bufferedCommand.length > 0) && empty) {
       if (sawSIGINT) {
@@ -399,7 +494,21 @@ function REPLServer(prompt,
   self.on('line', function(cmd) {
     debug('line %j', cmd);
     sawSIGINT = false;
-    var skipCatchall = false;
+
+    if (self.editorMode) {
+      self.bufferedCommand += cmd + '\n';
+
+      // code alignment
+      const matches = self._sawKeyPress ? cmd.match(/^\s+/) : null;
+      if (matches) {
+        const prefix = matches[0];
+        self.write(prefix);
+        self.line = prefix;
+        self.cursor = prefix.length;
+      }
+      self.memory(cmd);
+      return;
+    }
 
     // leading whitespaces in template literals should not be trimmed.
     if (self._inTemplateLiteral) {
@@ -410,43 +519,30 @@ function REPLServer(prompt,
 
     // Check to see if a REPL keyword was used. If it returns true,
     // display next prompt and return.
-    if (cmd && cmd.charAt(0) === '.' && isNaN(parseFloat(cmd))) {
-      var matches = cmd.match(/^\.([^\s]+)\s*(.*)$/);
-      var keyword = matches && matches[1];
-      var rest = matches && matches[2];
+    if (cmd && cmd.charAt(0) === '.' && cmd.charAt(1) !== '.' &&
+        isNaN(parseFloat(cmd))) {
+      const matches = cmd.match(/^\.([^\s]+)\s*(.*)$/);
+      const keyword = matches && matches[1];
+      const rest = matches && matches[2];
       if (self.parseREPLKeyword(keyword, rest) === true) {
         return;
-      } else {
+      } else if (!self.bufferedCommand) {
         self.outputStream.write('Invalid REPL keyword\n');
-        skipCatchall = true;
+        finish(null);
+        return;
       }
     }
 
-    if (!skipCatchall && (cmd || (!cmd && self.bufferedCommand))) {
-      var evalCmd = self.bufferedCommand + cmd;
-      if (/^\s*\{/.test(evalCmd) && /\}\s*$/.test(evalCmd)) {
-        // It's confusing for `{ a : 1 }` to be interpreted as a block
-        // statement rather than an object literal.  So, we first try
-        // to wrap it in parentheses, so that it will be interpreted as
-        // an expression.
-        evalCmd = '(' + evalCmd + ')\n';
-      } else {
-        // otherwise we just append a \n so that it will be either
-        // terminated, or continued onto the next expression if it's an
-        // unexpected end of input.
-        evalCmd = evalCmd + '\n';
-      }
+    const evalCmd = self.bufferedCommand + cmd + '\n';
 
-      debug('eval %j', evalCmd);
-      self.eval(evalCmd, self.context, 'repl', finish);
-    } else {
-      finish(null);
-    }
+    debug('eval %j', evalCmd);
+    self.eval(evalCmd, self.context, 'repl', finish);
 
     function finish(e, ret) {
       debug('finish', e, ret);
       self.memory(cmd);
 
+      self.wrappedCmd = false;
       if (e && !self.bufferedCommand && cmd.trim().startsWith('npm ')) {
         self.outputStream.write('npm should be run outside of the ' +
                                 'node repl, in your normal shell.\n' +
@@ -459,7 +555,8 @@ function REPLServer(prompt,
 
       // If error was SyntaxError and not JSON.parse error
       if (e) {
-        if (e instanceof Recoverable && !self.lineParser.shouldFail) {
+        if (e instanceof Recoverable && !self.lineParser.shouldFail &&
+            !sawCtrlD) {
           // Start buffering data like that:
           // {
           // ...  x: 1
@@ -475,6 +572,7 @@ function REPLServer(prompt,
       // Clear buffer if no SyntaxErrors
       self.lineParser.reset();
       self.bufferedCommand = '';
+      sawCtrlD = false;
 
       // If we got any output - print it (if no error)
       if (!e &&
@@ -483,7 +581,9 @@ function REPLServer(prompt,
           // the second argument to this function is there, print it.
           arguments.length === 2 &&
           (!self.ignoreUndefined || ret !== undefined)) {
-        self.context._ = ret;
+        if (!self.underscoreAssigned) {
+          self.last = ret;
+        }
         self.outputStream.write(self.writer(ret) + '\n');
       }
 
@@ -493,12 +593,59 @@ function REPLServer(prompt,
   });
 
   self.on('SIGCONT', function() {
-    self.displayPrompt(true);
+    if (self.editorMode) {
+      self.outputStream.write(`${self._initialPrompt}.editor\n`);
+      self.outputStream.write(
+        '// Entering editor mode (^D to finish, ^C to cancel)\n');
+      self.outputStream.write(`${self.bufferedCommand}\n`);
+      self.prompt(true);
+    } else {
+      self.displayPrompt(true);
+    }
   });
 
+  // Wrap readline tty to enable editor mode
+  const ttyWrite = self._ttyWrite.bind(self);
+  self._ttyWrite = (d, key) => {
+    key = key || {};
+    if (!self.editorMode || !self.terminal) {
+      ttyWrite(d, key);
+      return;
+    }
+
+    // editor mode
+    if (key.ctrl && !key.shift) {
+      switch (key.name) {
+        case 'd': // End editor mode
+          self.turnOffEditorMode();
+          sawCtrlD = true;
+          ttyWrite(d, { name: 'return' });
+          break;
+        case 'n': // Override next history item
+        case 'p': // Override previous history item
+          break;
+        default:
+          ttyWrite(d, key);
+      }
+    } else {
+      switch (key.name) {
+        case 'up':   // Override previous history item
+        case 'down': // Override next history item
+          break;
+        case 'tab':
+          // prevent double tab behavior
+          self._previousKey = null;
+          ttyWrite(d, key);
+          break;
+        default:
+          ttyWrite(d, key);
+      }
+    }
+  };
+
   self.displayPrompt();
 }
-inherits(REPLServer, rl.Interface);
+inherits(REPLServer, Interface);
 exports.REPLServer = REPLServer;
 
 exports.REPL_MODE_SLOPPY = Symbol('repl-sloppy');
@@ -524,6 +671,20 @@ exports.start = function(prompt,
   return repl;
 };
 
+REPLServer.prototype.close = function close() {
+  if (this.terminal && this._flushing && !this._closingOnFlush) {
+    this._closingOnFlush = true;
+    this.once('flushHistory', () =>
+      Interface.prototype.close.call(this)
+    );
+
+    return;
+  }
+  process.nextTick(() =>
+    Interface.prototype.close.call(this)
+  );
+};
+
 REPLServer.prototype.createContext = function() {
   var context;
   if (this.useGlobal) {
@@ -542,7 +703,7 @@ REPLServer.prototype.createContext = function() {
       return GLOBAL_OBJECT_PROPERTY_MAP[name] === undefined;
     }).forEach((name) => {
       Object.defineProperty(context, name,
-        Object.getOwnPropertyDescriptor(global, name));
+                            Object.getOwnPropertyDescriptor(global, name));
     });
   }
 
@@ -553,25 +714,23 @@ REPLServer.prototype.createContext = function() {
   context.module = module;
   context.require = require;
 
+
+  this.underscoreAssigned = false;
   this.lines = [];
   this.lines.level = [];
 
-  // make built-in modules available directly
-  // (loaded lazily)
-  exports._builtinLibs.forEach(function(name) {
-    Object.defineProperty(context, name, {
-      get: function() {
-        var lib = require(name);
-        context._ = context[name] = lib;
-        return lib;
-      },
-      // allow the creation of other globals with this name
-      set: function(val) {
-        delete context[name];
-        context[name] = val;
-      },
-      configurable: true
-    });
+  internalModule.addBuiltinLibsToObject(context);
+
+  Object.defineProperty(context, '_', {
+    configurable: true,
+    get: () => this.last,
+    set: (value) => {
+      this.last = value;
+      if (!this.underscoreAssigned) {
+        this.underscoreAssigned = true;
+        this.outputStream.write('Expression assignment to _ now disabled.\n');
+      }
+    }
   });
 
   return context;
@@ -604,6 +763,12 @@ REPLServer.prototype.setPrompt = function setPrompt(prompt) {
   REPLServer.super_.prototype.setPrompt.call(this, prompt);
 };
 
+REPLServer.prototype.turnOffEditorMode = function() {
+  this.editorMode = false;
+  this.setPrompt(this._initialPrompt);
+};
+
+
 // A stream to push an array into a REPL
 // used in REPLServer.complete
 function ArrayStream() {
@@ -622,7 +787,7 @@ ArrayStream.prototype.writable = true;
 ArrayStream.prototype.resume = function() {};
 ArrayStream.prototype.write = function() {};
 
-const requireRE = /\brequire\s*\(['"](([\w./-]+\/)?([\w./-]*))/;
+const requireRE = /\brequire\s*\(['"](([\w@./-]+\/)?([\w@./-]*))/;
 const simpleExpressionRE =
     /(([a-zA-Z_$](?:\w|\$)*)\.)*([a-zA-Z_$](?:\w|\$)*)\.?$/;
 
@@ -636,20 +801,24 @@ function filteredOwnPropertyNames(obj) {
   return Object.getOwnPropertyNames(obj).filter(intFilter);
 }
 
+REPLServer.prototype.complete = function() {
+  this.completer.apply(this, arguments);
+};
+
 // Provide a list of completions for the given leading text. This is
 // given to the readline interface for handling tab completion.
 //
 // Example:
 //  complete('var foo = util.')
-//    -> [['util.print', 'util.debug', 'util.log', 'util.inspect', 'util.pump'],
+//    -> [['util.print', 'util.debug', 'util.log', 'util.inspect'],
 //        'util.' ]
 //
 // Warning: This eval's code like "foo.bar.baz", so it will run property
 // getter code.
-REPLServer.prototype.complete = function(line, callback) {
+function complete(line, callback) {
   // There may be local variables to evaluate, try a nested REPL
   if (this.bufferedCommand !== undefined && this.bufferedCommand.length) {
-    // Get a new array of inputed lines
+    // Get a new array of inputted lines
     var tmp = this.lines.slice();
     // Kill off all function declarations to push all local variables into
     // global scope
@@ -660,12 +829,12 @@ REPLServer.prototype.complete = function(line, callback) {
     });
     var flat = new ArrayStream();         // make a new "input" stream
     var magic = new REPLServer('', flat); // make a nested REPL
+    replMap.set(magic, replMap.get(this));
     magic.context = magic.createContext();
     flat.run(tmp);                        // eval the flattened code
     // all this is only profitable if the nested REPL
     // does not have a bufferedCommand
     if (!magic.bufferedCommand) {
-      replMap.set(magic, replMap.get(this));
       return magic.complete(line, callback);
     }
   }
@@ -678,6 +847,7 @@ REPLServer.prototype.complete = function(line, callback) {
   var completeOn, i, group, c;
 
   // REPL commands (e.g. ".break").
+  var filter;
   var match = null;
   match = line.match(/^\s*\.(\w*)$/);
   if (match) {
@@ -693,13 +863,14 @@ REPLServer.prototype.complete = function(line, callback) {
     const exts = Object.keys(this.context.require.extensions);
     var indexRe = new RegExp('^index(' + exts.map(regexpEscape).join('|') +
                              ')$');
+    var versionedFileNamesRe = /-\d+\.\d+/;
 
     completeOn = match[1];
     var subdir = match[2] || '';
-    var filter = match[1];
+    filter = match[1];
     var dir, files, f, name, base, ext, abs, subfiles, s;
     group = [];
-    var paths = module.paths.concat(require('module').globalPaths);
+    var paths = module.paths.concat(Module.globalPaths);
     for (i = 0; i < paths.length; i++) {
       dir = path.resolve(paths[i], subdir);
       try {
@@ -711,7 +882,7 @@ REPLServer.prototype.complete = function(line, callback) {
         name = files[f];
         ext = path.extname(name);
         base = name.slice(0, -ext.length);
-        if (base.match(/-\d+\.\d+(\.\d+)?/) || name === '.npm') {
+        if (versionedFileNamesRe.test(base) || name === '.npm') {
           // Exclude versioned names that 'npm' installs.
           continue;
         }
@@ -755,7 +926,7 @@ REPLServer.prototype.complete = function(line, callback) {
   //   spam.eggs.<|>  # completions for 'spam.eggs' with filter ''
   //   foo<|>         # all scope vars with filter 'foo'
   //   foo.<|>        # completions for 'foo' with filter ''
-  } else if (line.length === 0 || line[line.length - 1].match(/\w|\.|\$/)) {
+  } else if (line.length === 0 || /\w|\.|\$/.test(line[line.length - 1])) {
     match = simpleExpressionRE.exec(line);
     if (line.length === 0 || match) {
       var expr;
@@ -802,7 +973,8 @@ REPLServer.prototype.complete = function(line, callback) {
           });
         }
       } else {
-        this.eval(expr, this.context, 'repl', function(e, obj) {
+        const evalExpr = `try { ${expr} } catch (e) {}`;
+        this.eval(evalExpr, this.context, 'repl', function(e, obj) {
           // if (e) console.log(e);
 
           if (obj != null) {
@@ -904,8 +1076,41 @@ REPLServer.prototype.complete = function(line, callback) {
 
     callback(null, [completions || [], completeOn]);
   }
-};
+}
 
+function longestCommonPrefix(arr = []) {
+  const cnt = arr.length;
+  if (cnt === 0) return '';
+  if (cnt === 1) return arr[0];
+
+  const first = arr[0];
+  // complexity: O(m * n)
+  for (var m = 0; m < first.length; m++) {
+    const c = first[m];
+    for (var n = 1; n < cnt; n++) {
+      const entry = arr[n];
+      if (m >= entry.length || c !== entry[m]) {
+        return first.substring(0, m);
+      }
+    }
+  }
+  return first;
+}
+
+REPLServer.prototype.completeOnEditorMode = (callback) => (err, results) => {
+  if (err) return callback(err);
+
+  const [completions, completeOn = ''] = results;
+  const prefixLength = completeOn.length;
+
+  if (prefixLength === 0) return callback(null, [[], completeOn]);
+
+  const isNotEmpty = (v) => v.length > 0;
+  const trimCompleteOnPrefix = (v) => v.substring(prefixLength);
+  const data = completions.filter(isNotEmpty).map(trimCompleteOnPrefix);
+
+  callback(null, [[`${completeOn}${longestCommonPrefix(data)}`], completeOn]);
+};
 
 /**
  * Used to parse and execute the Node REPL commands.
@@ -927,7 +1132,7 @@ REPLServer.prototype.defineCommand = function(keyword, cmd) {
   if (typeof cmd === 'function') {
     cmd = {action: cmd};
   } else if (typeof cmd.action !== 'function') {
-    throw new Error('bad argument, action must be a function');
+    throw new Error('Bad argument, "action" command must be a function');
   }
   this.commands[keyword] = cmd;
 };
@@ -973,7 +1178,7 @@ REPLServer.prototype.memory = function memory(cmd) {
           self.lines.level.push({
             line: self.lines.length - 1,
             depth: depth,
-            isFunction: /\s*function\s*/.test(cmd)
+            isFunction: /\bfunction\b/.test(cmd)
           });
         } else if (depth < 0) {
           // going... up.
@@ -999,7 +1204,7 @@ REPLServer.prototype.memory = function memory(cmd) {
     // self.lines.level.length === 0
     // TODO? keep a log of level so that any syntax breaking lines can
     // be cleared on .break and in the case of a syntax error?
-    // TODO? if a log was kept, then I could clear the bufferedComand and
+    // TODO? if a log was kept, then I could clear the bufferedCommand and
     // eval these lines and throw the syntax error
   } else {
     self.lines.level = [];
@@ -1013,12 +1218,13 @@ function addStandardGlobals(completionGroups, filter) {
   // Common keywords. Exclude for completion on the empty string, b/c
   // they just get in the way.
   if (filter) {
-    completionGroups.push(['break', 'case', 'catch', 'const',
-      'continue', 'debugger', 'default', 'delete', 'do', 'else',
-      'export', 'false', 'finally', 'for', 'function', 'if',
-      'import', 'in', 'instanceof', 'let', 'new', 'null', 'return',
-      'switch', 'this', 'throw', 'true', 'try', 'typeof', 'undefined',
-      'var', 'void', 'while', 'with', 'yield']);
+    completionGroups.push([
+      'break', 'case', 'catch', 'const', 'continue', 'debugger', 'default',
+      'delete', 'do', 'else', 'export', 'false', 'finally', 'for', 'function',
+      'if', 'import', 'in', 'instanceof', 'let', 'new', 'null', 'return',
+      'switch', 'this', 'throw', 'true', 'try', 'typeof', 'undefined', 'var',
+      'void', 'while', 'with', 'yield'
+    ]);
   }
 }
 
@@ -1059,12 +1265,18 @@ function defineDefaultCommands(repl) {
   });
 
   repl.defineCommand('help', {
-    help: 'Show repl options',
+    help: 'Print this help message',
     action: function() {
-      var self = this;
-      Object.keys(this.commands).sort().forEach(function(name) {
-        var cmd = self.commands[name];
-        self.outputStream.write(name + '\t' + (cmd.help || '') + '\n');
+      const names = Object.keys(this.commands).sort();
+      const longestNameLength = names.reduce(
+        (max, name) => Math.max(max, name.length),
+        0
+      );
+      names.forEach((name) => {
+        const cmd = this.commands[name];
+        const spaces = ' '.repeat(longestNameLength - name.length + 3);
+        const line = '.' + name + (cmd.help ? spaces + cmd.help : '') + '\n';
+        this.outputStream.write(line);
       });
       this.displayPrompt();
     }
@@ -1108,6 +1320,17 @@ function defineDefaultCommands(repl) {
       this.displayPrompt();
     }
   });
+
+  repl.defineCommand('editor', {
+    help: 'Enter editor mode',
+    action() {
+      if (!this.terminal) return;
+      this.editorMode = true;
+      REPLServer.super_.prototype.setPrompt.call(this, '');
+      this.outputStream.write(
+        '// Entering editor mode (^D to finish, ^C to cancel)\n');
+    }
+  });
 }
 
 function regexpEscape(s) {
@@ -1143,6 +1366,11 @@ REPLServer.prototype.convertToContext = function(cmd) {
   return cmd;
 };
 
+function bailOnIllegalToken(parser) {
+  return parser._literal === null &&
+         !parser.blockComment &&
+         !parser.regExpLiteral;
+}
 
 // If the error is that we've unexpectedly ended the input,
 // then let the user try to recover by adding more input.
@@ -1154,7 +1382,14 @@ function isRecoverableError(e, self) {
       self._inTemplateLiteral = true;
       return true;
     }
-    return /^(Unexpected end of input|Unexpected token)/.test(message);
+
+    if (message.startsWith('Unexpected end of input') ||
+        message.startsWith('missing ) after argument list') ||
+        message.startsWith('Unexpected token'))
+      return true;
+
+    if (message === 'Invalid or unexpected token')
+      return !bailOnIllegalToken(self.lineParser);
   }
   return false;
 }
@@ -1163,3 +1398,4 @@ function Recoverable(err) {
   this.err = err;
 }
 inherits(Recoverable, SyntaxError);
+exports.Recoverable = Recoverable;
