@@ -7,7 +7,54 @@ const binding = process.binding('util');
 
 const isError = internalUtil.isError;
 
+const inspectDefaultOptions = Object.seal({
+  showHidden: false,
+  depth: 2,
+  colors: false,
+  customInspect: true,
+  showProxy: false,
+  maxArrayLength: 100,
+  breakLength: 60
+});
+
+const numbersOnlyRE = /^\d+$/;
+
 var Debug;
+var simdFormatters;
+
+// SIMD is only available when --harmony_simd is specified on the command line
+// and the set of available types differs between v5 and v6, that's why we use
+// a map to look up and store the formatters.  It also provides a modicum of
+// protection against users monkey-patching the SIMD object.
+if (typeof global.SIMD === 'object' && global.SIMD !== null) {
+  simdFormatters = new Map();
+
+  const make =
+    (extractLane, count) => (ctx, value, recurseTimes, visibleKeys, keys) => {
+      const output = new Array(count);
+      for (var i = 0; i < count; i += 1)
+        output[i] = formatPrimitive(ctx, extractLane(value, i));
+      return output;
+    };
+
+  const countPerType = {
+    Bool16x8: 8,
+    Bool32x4: 4,
+    Bool8x16: 16,
+    Float32x4: 4,
+    Int16x8: 8,
+    Int32x4: 4,
+    Int8x16: 16,
+    Uint16x8: 8,
+    Uint32x4: 4,
+    Uint8x16: 16
+  };
+
+  for (const key in countPerType) {
+    const type = global.SIMD[key];
+    simdFormatters.set(type, make(type.extractLane, countPerType[key]));
+  }
+}
 
 function tryStringify(arg) {
   try {
@@ -125,25 +172,31 @@ function inspect(obj, opts) {
     stylize: stylizeNoColor
   };
   // legacy...
-  if (arguments.length >= 3) ctx.depth = arguments[2];
-  if (arguments.length >= 4) ctx.colors = arguments[3];
+  if (arguments[2] !== undefined) ctx.depth = arguments[2];
+  if (arguments[3] !== undefined) ctx.colors = arguments[3];
   if (typeof opts === 'boolean') {
     // legacy...
     ctx.showHidden = opts;
-  } else if (opts) {
-    // got an "options" object
-    exports._extend(ctx, opts);
   }
-  // set default options
-  if (ctx.showHidden === undefined) ctx.showHidden = false;
-  if (ctx.depth === undefined) ctx.depth = 2;
-  if (ctx.colors === undefined) ctx.colors = false;
-  if (ctx.customInspect === undefined) ctx.customInspect = true;
+  // Set default and user-specified options
+  ctx = Object.assign({}, inspect.defaultOptions, ctx, opts);
   if (ctx.colors) ctx.stylize = stylizeWithColor;
+  if (ctx.maxArrayLength === null) ctx.maxArrayLength = Infinity;
   return formatValue(ctx, obj, ctx.depth);
 }
-exports.inspect = inspect;
 
+Object.defineProperty(inspect, 'defaultOptions', {
+  get: function() {
+    return inspectDefaultOptions;
+  },
+  set: function(options) {
+    if (options === null || typeof options !== 'object') {
+      throw new TypeError('"options" must be an object');
+    }
+    Object.assign(inspectDefaultOptions, options);
+    return inspectDefaultOptions;
+  }
+});
 
 // http://en.wikipedia.org/wiki/ANSI_escape_code#graphics
 inspect.colors = {
@@ -176,6 +229,10 @@ inspect.styles = {
   'regexp': 'red'
 };
 
+const customInspectSymbol = internalUtil.customInspectSymbol;
+
+exports.inspect = inspect;
+exports.inspect.custom = customInspectSymbol;
 
 function stylizeWithColor(str, styleType) {
   var style = inspect.styles[styleType];
@@ -195,7 +252,7 @@ function stylizeNoColor(str, styleType) {
 
 
 function arrayToHash(array) {
-  var hash = {};
+  var hash = Object.create(null);
 
   for (var i = 0; i < array.length; i++) {
     var val = array[i];
@@ -231,30 +288,77 @@ function ensureDebugIsInitialized() {
 
 
 function inspectPromise(p) {
-  ensureDebugIsInitialized();
   // Only create a mirror if the object is a Promise.
   if (!binding.isPromise(p))
     return null;
+  ensureDebugIsInitialized();
   const mirror = Debug.MakeMirror(p, true);
   return {status: mirror.status(), value: mirror.promiseValue().value_};
 }
 
 
 function formatValue(ctx, value, recurseTimes) {
+  if (ctx.showProxy &&
+      ((typeof value === 'object' && value !== null) ||
+       typeof value === 'function')) {
+    var proxy = undefined;
+    var proxyCache = ctx.proxyCache;
+    if (!proxyCache)
+      proxyCache = ctx.proxyCache = new Map();
+    // Determine if we've already seen this object and have
+    // determined that it either is or is not a proxy.
+    if (proxyCache.has(value)) {
+      // We've seen it, if the value is not undefined, it's a Proxy.
+      proxy = proxyCache.get(value);
+    } else {
+      // Haven't seen it. Need to check.
+      // If it's not a Proxy, this will return undefined.
+      // Otherwise, it'll return an array. The first item
+      // is the target, the second item is the handler.
+      // We ignore (and do not return) the Proxy isRevoked property.
+      proxy = binding.getProxyDetails(value);
+      if (proxy) {
+        // We know for a fact that this isn't a Proxy.
+        // Mark it as having already been evaluated.
+        // We do this because this object is passed
+        // recursively to formatValue below in order
+        // for it to get proper formatting, and because
+        // the target and handle objects also might be
+        // proxies... it's unfortunate but necessary.
+        proxyCache.set(proxy, undefined);
+      }
+      // If the object is not a Proxy, then this stores undefined.
+      // This tells the code above that we've already checked and
+      // ruled it out. If the object is a proxy, this caches the
+      // results of the getProxyDetails call.
+      proxyCache.set(value, proxy);
+    }
+    if (proxy) {
+      return 'Proxy ' + formatValue(ctx, proxy, recurseTimes);
+    }
+  }
+
   // Provide a hook for user-specified inspect functions.
   // Check that value is an object with an inspect function on it
-  if (ctx.customInspect &&
-      value &&
-      typeof value.inspect === 'function' &&
-      // Filter out the util module, it's inspect function is special
-      value.inspect !== exports.inspect &&
-      // Also filter out any prototype objects using the circular check.
-      !(value.constructor && value.constructor.prototype === value)) {
-    var ret = value.inspect(recurseTimes, ctx);
-    if (typeof ret !== 'string') {
-      ret = formatValue(ctx, ret, recurseTimes);
+  if (ctx.customInspect && value) {
+    const maybeCustomInspect = value[customInspectSymbol] || value.inspect;
+
+    if (typeof maybeCustomInspect === 'function' &&
+        // Filter out the util module, its inspect function is special
+        maybeCustomInspect !== exports.inspect &&
+        // Also filter out any prototype objects using the circular check.
+        !(value.constructor && value.constructor.prototype === value)) {
+      let ret = maybeCustomInspect.call(value, recurseTimes, ctx);
+
+      // If the custom inspection method returned `this`, don't go into
+      // infinite recursion.
+      if (ret !== value) {
+        if (typeof ret !== 'string') {
+          ret = formatValue(ctx, ret, recurseTimes);
+        }
+        return ret;
+      }
     }
-    return ret;
   }
 
   // Primitive types cannot have properties
@@ -298,13 +402,17 @@ function formatValue(ctx, value, recurseTimes) {
   if (keys.length === 0) {
     if (typeof value === 'function') {
       return ctx.stylize(`[Function${value.name ? `: ${value.name}` : ''}]`,
-        'special');
+                         'special');
     }
     if (isRegExp(value)) {
       return ctx.stylize(RegExp.prototype.toString.call(value), 'regexp');
     }
     if (isDate(value)) {
-      return ctx.stylize(Date.prototype.toString.call(value), 'date');
+      if (Number.isNaN(value.getTime())) {
+        return ctx.stylize(value.toString(), 'date');
+      } else {
+        return ctx.stylize(Date.prototype.toISOString.call(value), 'date');
+      }
     }
     if (isError(value)) {
       return formatError(value);
@@ -326,10 +434,18 @@ function formatValue(ctx, value, recurseTimes) {
       formatted = formatPrimitiveNoColor(ctx, raw);
       return ctx.stylize(`[Boolean: ${formatted}]`, 'boolean');
     }
+    // Fast path for ArrayBuffer and SharedArrayBuffer.
+    // Can't do the same for DataView because it has a non-primitive
+    // .buffer property that we need to recurse for.
+    if (binding.isArrayBuffer(value) || binding.isSharedArrayBuffer(value)) {
+      return `${getConstructorOf(value).name}` +
+             ` { byteLength: ${formatNumber(ctx, value.byteLength)} }`;
+    }
   }
 
   var constructor = getConstructorOf(value);
-  var base = '', empty = false, braces, formatter;
+  var base = '', empty = false, braces;
+  var formatter = formatObject;
 
   // We can't compare constructors for various objects using a comparison like
   // `constructor === Array` because the object could have come from a different
@@ -359,12 +475,36 @@ function formatValue(ctx, value, recurseTimes) {
       keys.unshift('size');
     empty = value.size === 0;
     formatter = formatMap;
+  } else if (binding.isArrayBuffer(value) ||
+             binding.isSharedArrayBuffer(value)) {
+    braces = ['{', '}'];
+    keys.unshift('byteLength');
+    visibleKeys.byteLength = true;
+  } else if (binding.isDataView(value)) {
+    braces = ['{', '}'];
+    // .buffer goes last, it's not a primitive like the others.
+    keys.unshift('byteLength', 'byteOffset', 'buffer');
+    visibleKeys.byteLength = true;
+    visibleKeys.byteOffset = true;
+    visibleKeys.buffer = true;
+  } else if (binding.isTypedArray(value)) {
+    braces = ['[', ']'];
+    formatter = formatTypedArray;
+    if (ctx.showHidden) {
+      // .buffer goes last, it's not a primitive like the others.
+      keys.unshift('BYTES_PER_ELEMENT',
+                   'length',
+                   'byteLength',
+                   'byteOffset',
+                   'buffer');
+    }
   } else {
     var promiseInternals = inspectPromise(value);
     if (promiseInternals) {
       braces = ['{', '}'];
       formatter = formatPromise;
     } else {
+      let maybeSimdFormatter;
       if (binding.isMapIterator(value)) {
         constructor = { name: 'MapIterator' };
         braces = ['{', '}'];
@@ -375,13 +515,17 @@ function formatValue(ctx, value, recurseTimes) {
         braces = ['{', '}'];
         empty = false;
         formatter = formatCollectionIterator;
+      } else if (simdFormatters &&
+                 typeof constructor === 'function' &&
+                 (maybeSimdFormatter = simdFormatters.get(constructor))) {
+        braces = ['[', ']'];
+        formatter = maybeSimdFormatter;
       } else {
         // Unset the constructor to prevent "Object {...}" for ordinary objects.
         if (constructor && constructor.name === 'Object')
           constructor = null;
         braces = ['{', '}'];
         empty = true;  // No other data than keys.
-        formatter = formatObject;
       }
     }
   }
@@ -400,7 +544,7 @@ function formatValue(ctx, value, recurseTimes) {
 
   // Make dates with properties first say the date
   if (isDate(value)) {
-    base = ' ' + Date.prototype.toUTCString.call(value);
+    base = ' ' + Date.prototype.toISOString.call(value);
   }
 
   // Make error with message first say the error
@@ -448,7 +592,15 @@ function formatValue(ctx, value, recurseTimes) {
 
   ctx.seen.pop();
 
-  return reduceToSingleString(output, base, braces);
+  return reduceToSingleString(output, base, braces, ctx.breakLength);
+}
+
+
+function formatNumber(ctx, value) {
+  // Format -0 as '-0'. Strict equality won't distinguish 0 from -0.
+  if (Object.is(value, -0))
+    return ctx.stylize('-0', 'number');
+  return ctx.stylize('' + value, 'number');
 }
 
 
@@ -471,13 +623,8 @@ function formatPrimitive(ctx, value) {
                  '\'';
     return ctx.stylize(simple, 'string');
   }
-  if (type === 'number') {
-    // Format -0 as '-0'. Strict equality won't distinguish 0 from -0,
-    // so instead we use the fact that 1 / -0 < 0 whereas 1 / 0 > 0 .
-    if (value === 0 && 1 / value < 0)
-      return ctx.stylize('-0', 'number');
-    return ctx.stylize('' + value, 'number');
-  }
+  if (type === 'number')
+    return formatNumber(ctx, value);
   if (type === 'boolean')
     return ctx.stylize('' + value, 'boolean');
   // es6 symbol primitive
@@ -496,7 +643,7 @@ function formatPrimitiveNoColor(ctx, value) {
 
 
 function formatError(value) {
-  return `[${Error.prototype.toString.call(value)}]`;
+  return value.stack || `[${Error.prototype.toString.call(value)}]`;
 }
 
 
@@ -509,24 +656,48 @@ function formatObject(ctx, value, recurseTimes, visibleKeys, keys) {
 
 function formatArray(ctx, value, recurseTimes, visibleKeys, keys) {
   var output = [];
-  for (var i = 0, l = value.length; i < l; ++i) {
+  const maxLength = Math.min(Math.max(0, ctx.maxArrayLength), value.length);
+  const remaining = value.length - maxLength;
+  for (var i = 0; i < maxLength; ++i) {
     if (hasOwnProperty(value, String(i))) {
       output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
-          String(i), true));
+                                 String(i), true));
     } else {
       output.push('');
     }
   }
+  if (remaining > 0) {
+    output.push(`... ${remaining} more item${remaining > 1 ? 's' : ''}`);
+  }
   keys.forEach(function(key) {
-    if (typeof key === 'symbol' || !key.match(/^\d+$/)) {
+    if (typeof key === 'symbol' || !numbersOnlyRE.test(key)) {
       output.push(formatProperty(ctx, value, recurseTimes, visibleKeys,
-          key, true));
+                                 key, true));
     }
   });
   return output;
 }
 
 
+function formatTypedArray(ctx, value, recurseTimes, visibleKeys, keys) {
+  const maxLength = Math.min(Math.max(0, ctx.maxArrayLength), value.length);
+  const remaining = value.length - maxLength;
+  var output = new Array(maxLength);
+  for (var i = 0; i < maxLength; ++i)
+    output[i] = formatNumber(ctx, value[i]);
+  if (remaining > 0) {
+    output.push(`... ${remaining} more item${remaining > 1 ? 's' : ''}`);
+  }
+  for (const key of keys) {
+    if (typeof key === 'symbol' || !numbersOnlyRE.test(key)) {
+      output.push(
+        formatProperty(ctx, value, recurseTimes, visibleKeys, key, true));
+    }
+  }
+  return output;
+}
+
+
 function formatSet(ctx, value, recurseTimes, visibleKeys, keys) {
   var output = [];
   value.forEach(function(v) {
@@ -632,11 +803,11 @@ function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
     }
   }
   if (name === undefined) {
-    if (array && key.match(/^\d+$/)) {
+    if (array && numbersOnlyRE.test(key)) {
       return str;
     }
     name = JSON.stringify('' + key);
-    if (name.match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)) {
+    if (/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/.test(name)) {
       name = name.substr(1, name.length - 2);
       name = ctx.stylize(name, 'name');
     } else {
@@ -652,12 +823,12 @@ function formatProperty(ctx, value, recurseTimes, visibleKeys, key, array) {
 }
 
 
-function reduceToSingleString(output, base, braces) {
+function reduceToSingleString(output, base, braces, breakLength) {
   var length = output.reduce(function(prev, cur) {
     return prev + cur.replace(/\u001b\[\d\d?m/g, '').length + 1;
   }, 0);
 
-  if (length > 60) {
+  if (length > breakLength) {
     return braces[0] +
            // If the opening "brace" is too large, like in the case of "Set {",
            // we need to force the first item to be on the next line or the
@@ -739,7 +910,6 @@ exports.isPrimitive = isPrimitive;
 
 exports.isBuffer = Buffer.isBuffer;
 
-
 function pad(n) {
   return n < 10 ? '0' + n.toString(10) : n.toString(10);
 }
@@ -782,38 +952,31 @@ exports.log = function() {
 exports.inherits = function(ctor, superCtor) {
 
   if (ctor === undefined || ctor === null)
-    throw new TypeError('The constructor to `inherits` must not be ' +
-                        'null or undefined.');
+    throw new TypeError('The constructor to "inherits" must not be ' +
+                        'null or undefined');
 
   if (superCtor === undefined || superCtor === null)
-    throw new TypeError('The super constructor to `inherits` must not ' +
-                        'be null or undefined.');
+    throw new TypeError('The super constructor to "inherits" must not ' +
+                        'be null or undefined');
 
   if (superCtor.prototype === undefined)
-    throw new TypeError('The super constructor to `inherits` must ' +
-                        'have a prototype.');
+    throw new TypeError('The super constructor to "inherits" must ' +
+                        'have a prototype');
 
   ctor.super_ = superCtor;
-  ctor.prototype = Object.create(superCtor.prototype, {
-    constructor: {
-      value: ctor,
-      enumerable: false,
-      writable: true,
-      configurable: true
-    }
-  });
+  Object.setPrototypeOf(ctor.prototype, superCtor.prototype);
 };
 
-exports._extend = function(origin, add) {
-  // Don't do anything if add isn't an object
-  if (add === null || typeof add !== 'object') return origin;
+exports._extend = function(target, source) {
+  // Don't do anything if source isn't an object
+  if (source === null || typeof source !== 'object') return target;
 
-  var keys = Object.keys(add);
+  var keys = Object.keys(source);
   var i = keys.length;
   while (i--) {
-    origin[keys[i]] = add[keys[i]];
+    target[keys[i]] = source[keys[i]];
   }
-  return origin;
+  return target;
 };
 
 function hasOwnProperty(obj, prop) {
@@ -823,18 +986,6 @@ function hasOwnProperty(obj, prop) {
 
 // Deprecated old stuff.
 
-exports.p = internalUtil.deprecate(function() {
-  for (var i = 0, len = arguments.length; i < len; ++i) {
-    console.error(exports.inspect(arguments[i]));
-  }
-}, 'util.p is deprecated. Use console.error instead.');
-
-
-exports.exec = internalUtil.deprecate(function() {
-  return require('child_process').exec.apply(this, arguments);
-}, 'util.exec is deprecated. Use child_process.exec instead.');
-
-
 exports.print = internalUtil.deprecate(function() {
   for (var i = 0, len = arguments.length; i < len; ++i) {
     process.stdout.write(String(arguments[i]));
@@ -861,44 +1012,6 @@ exports.error = internalUtil.deprecate(function(x) {
 }, 'util.error is deprecated. Use console.error instead.');
 
 
-exports.pump = internalUtil.deprecate(function(readStream, writeStream, cb) {
-  var callbackCalled = false;
-
-  function call(a, b, c) {
-    if (cb && !callbackCalled) {
-      cb(a, b, c);
-      callbackCalled = true;
-    }
-  }
-
-  readStream.addListener('data', function(chunk) {
-    if (writeStream.write(chunk) === false) readStream.pause();
-  });
-
-  writeStream.addListener('drain', function() {
-    readStream.resume();
-  });
-
-  readStream.addListener('end', function() {
-    writeStream.end();
-  });
-
-  readStream.addListener('close', function() {
-    call();
-  });
-
-  readStream.addListener('error', function(err) {
-    writeStream.end();
-    call(err);
-  });
-
-  writeStream.addListener('error', function(err) {
-    readStream.destroy();
-    call(err);
-  });
-}, 'util.pump is deprecated. Use readableStream.pipe instead.');
-
-
 exports._errnoException = function(err, syscall, original) {
   var errname = uv.errname(err);
   var message = `${syscall} ${errname}`;
