@@ -1,6 +1,8 @@
 #ifndef SRC_BASE_OBJECT_H_
 #define SRC_BASE_OBJECT_H_
 
+#if defined(NODE_WANT_INTERNALS) && NODE_WANT_INTERNALS
+
 #include "v8.h"
 
 namespace node {
@@ -9,8 +11,8 @@ class Environment;
 
 class BaseObject {
  public:
-  BaseObject(Environment* env, v8::Local<v8::Object> handle);
-  virtual ~BaseObject();
+  inline BaseObject(Environment* env, v8::Local<v8::Object> handle);
+  inline virtual ~BaseObject();
 
   // Returns the wrapped object.  Returns an empty handle when
   // persistent.IsEmpty() is true.
@@ -40,12 +42,14 @@ class BaseObject {
 
   template <typename Type>
   static inline void WeakCallback(
-      const v8::WeakCallbackData<v8::Object, Type>& data);
+      const v8::WeakCallbackInfo<Type>& data);
 
-  v8::Persistent<v8::Object> handle_;
+  v8::Persistent<v8::Object> persistent_handle_;
   Environment* env_;
 };
 
 }  // namespace node
 
+#endif  // defined(NODE_WANT_INTERNALS) && NODE_WANT_INTERNALS
+
 #endif  // SRC_BASE_OBJECT_H_
