@@ -1,6 +1,8 @@
 #ifndef SRC_NODE_CRYPTO_BIO_H_
 #define SRC_NODE_CRYPTO_BIO_H_
 
+#if defined(NODE_WANT_INTERNALS) && NODE_WANT_INTERNALS
+
 #include "openssl/bio.h"
 #include "env.h"
 #include "env-inl.h"
@@ -9,6 +11,7 @@
 #include "v8.h"
 
 namespace node {
+namespace crypto {
 
 class NodeBIO {
  public:
@@ -133,6 +136,9 @@ class NodeBIO {
   Buffer* write_head_;
 };
 
+}  // namespace crypto
 }  // namespace node
 
+#endif  // defined(NODE_WANT_INTERNALS) && NODE_WANT_INTERNALS
+
 #endif  // SRC_NODE_CRYPTO_BIO_H_
