@@ -1,6 +1,8 @@
 #ifndef SRC_NODE_DTRACE_H_
 #define SRC_NODE_DTRACE_H_
 
+#if defined(NODE_WANT_INTERNALS) && NODE_WANT_INTERNALS
+
 #include "node.h"
 #include "v8.h"
 #include "env.h"
@@ -58,4 +60,6 @@ void InitDTrace(Environment* env, v8::Local<v8::Object> target);
 
 }  // namespace node
 
+#endif  // defined(NODE_WANT_INTERNALS) && NODE_WANT_INTERNALS
+
 #endif  // SRC_NODE_DTRACE_H_
