@@ -1,32 +1,33 @@
 'use strict';
 const common = require('../common');
+if (common.inFreeBSDJail)
+  common.skip('in a FreeBSD jail');
+
 const assert = require('assert');
 const dgram = require('dgram');
 const util = require('util');
 const networkInterfaces = require('os').networkInterfaces();
-const Buffer = require('buffer').Buffer;
 const fork = require('child_process').fork;
 const LOCAL_BROADCAST_HOST = '255.255.255.255';
 const TIMEOUT = common.platformTimeout(5000);
 const messages = [
-  new Buffer('First message to send'),
-  new Buffer('Second message to send'),
-  new Buffer('Third message to send'),
-  new Buffer('Fourth message to send')
+  Buffer.from('First message to send'),
+  Buffer.from('Second message to send'),
+  Buffer.from('Third message to send'),
+  Buffer.from('Fourth message to send')
 ];
 
-if (common.inFreeBSDJail) {
-  common.skip('in a FreeBSD jail');
-  return;
-}
+let bindAddress = null;
 
-// take the first non-internal interface as the address for binding
-get_bindAddress: for (var name in networkInterfaces) {
-  var interfaces = networkInterfaces[name];
-  for (var i = 0; i < interfaces.length; i++) {
-    var localInterface = interfaces[i];
+// Take the first non-internal interface as the address for binding.
+// Ideally, this should check for whether or not an interface is set up for
+// BROADCAST and favor internal/private interfaces.
+get_bindAddress: for (const name in networkInterfaces) {
+  const interfaces = networkInterfaces[name];
+  for (let i = 0; i < interfaces.length; i++) {
+    const localInterface = interfaces[i];
     if (!localInterface.internal && localInterface.family === 'IPv4') {
-      var bindAddress = localInterface.address;
+      bindAddress = localInterface.address;
       break get_bindAddress;
     }
   }
@@ -54,9 +55,9 @@ if (process.argv[2] !== 'child') {
   }, TIMEOUT);
 
   //launch child processes
-  for (var x = 0; x < listeners; x++) {
+  for (let x = 0; x < listeners; x++) {
     (function() {
-      var worker = fork(process.argv[1], ['child']);
+      const worker = fork(process.argv[1], ['child']);
       workers[worker.pid] = worker;
 
       worker.messagesReceived = [];
@@ -66,7 +67,7 @@ if (process.argv[2] !== 'child') {
         // don't consider this the true death if the worker
         // has finished successfully
         // or if the exit code is 0
-        if (worker.isDone || code == 0) {
+        if (worker.isDone || code === 0) {
           return;
         }
 
@@ -111,12 +112,12 @@ if (process.argv[2] !== 'child') {
                           'messages. Will now compare.');
 
             Object.keys(workers).forEach(function(pid) {
-              var worker = workers[pid];
+              const worker = workers[pid];
 
-              var count = 0;
+              let count = 0;
 
               worker.messagesReceived.forEach(function(buf) {
-                for (var i = 0; i < messages.length; ++i) {
+                for (let i = 0; i < messages.length; ++i) {
                   if (buf.toString() === messages[i].toString()) {
                     count++;
                     break;
@@ -128,8 +129,11 @@ if (process.argv[2] !== 'child') {
                             worker.pid,
                             count);
 
-              assert.equal(count, messages.length,
-                           'A worker received an invalid multicast message');
+              assert.strictEqual(
+                count,
+                messages.length,
+                'A worker received an invalid multicast message'
+              );
             });
 
             clearTimeout(timer);
@@ -141,7 +145,7 @@ if (process.argv[2] !== 'child') {
     })(x);
   }
 
-  var sendSocket = dgram.createSocket({
+  const sendSocket = dgram.createSocket({
     type: 'udp4',
     reuseAddr: true
   });
@@ -158,7 +162,7 @@ if (process.argv[2] !== 'child') {
   });
 
   sendSocket.sendNext = function() {
-    var buf = messages[i++];
+    const buf = messages[i++];
 
     if (!buf) {
       try { sendSocket.close(); } catch (e) {}
@@ -184,15 +188,15 @@ if (process.argv[2] !== 'child') {
 
   function killChildren(children) {
     Object.keys(children).forEach(function(key) {
-      var child = children[key];
+      const child = children[key];
       child.kill();
     });
   }
 }
 
 if (process.argv[2] === 'child') {
-  var receivedMessages = [];
-  var listenSocket = dgram.createSocket({
+  const receivedMessages = [];
+  const listenSocket = dgram.createSocket({
     type: 'udp4',
     reuseAddr: true
   });
@@ -208,9 +212,9 @@ if (process.argv[2] === 'child') {
 
     receivedMessages.push(buf);
 
-    process.send({ message: buf.toString() });
+    process.send({message: buf.toString()});
 
-    if (receivedMessages.length == messages.length) {
+    if (receivedMessages.length === messages.length) {
       process.nextTick(function() {
         listenSocket.close();
       });
@@ -227,7 +231,7 @@ if (process.argv[2] === 'child') {
   });
 
   listenSocket.on('listening', function() {
-    process.send({ listening: true });
+    process.send({listening: true});
   });
 
   listenSocket.bind(common.PORT);
