@@ -1,8 +1,7 @@
 'use strict';
 const common = require('../common');
-var assert = require('assert');
-var spawn = require('child_process').spawn;
-var fork = require('child_process').fork;
+const assert = require('assert');
+const { fork, spawn } = require('child_process');
 
 // Fork, then spawn. The spawned process should not hang.
 switch (process.argv[2] || '') {
@@ -16,10 +15,9 @@ switch (process.argv[2] || '') {
   case 'spawn':
     break;
   default:
-    assert(0);
+    common.fail();
 }
 
 function checkExit(statusCode) {
-  assert.equal(statusCode, 0);
-  process.nextTick(process.exit);
+  assert.strictEqual(statusCode, 0);
 }
