@@ -1,11 +1,11 @@
 'use strict';
 const common = require('../common');
-var assert = require('assert');
-var fork = require('child_process').fork;
-var args = ['foo', 'bar'];
+const assert = require('assert');
+const fork = require('child_process').fork;
+const args = ['foo', 'bar'];
 
-var n = fork(common.fixturesDir + '/child-process-spawn-node.js', args);
-assert.deepEqual(args, ['foo', 'bar']);
+const n = fork(`${common.fixturesDir}/child-process-spawn-node.js`, args);
+assert.deepStrictEqual(args, ['foo', 'bar']);
 
 n.on('message', function(m) {
   console.log('PARENT got message:', m);
