@@ -1,19 +1,19 @@
 'use strict';
+const common = require('../common');
 const assert = require('assert');
 const child_process = require('child_process');
 const spawn = child_process.spawn;
 const fork = child_process.fork;
 const execFile = child_process.execFile;
-const common = require('../common');
 const cmd = common.isWindows ? 'rundll32' : 'ls';
 const invalidcmd = 'hopefully_you_dont_have_this_on_your_machine';
 const invalidArgsMsg = /Incorrect value of args option/;
-const invalidOptionsMsg = /options argument must be an object/;
-const empty = common.fixturesDir + '/empty.js';
+const invalidOptionsMsg = /"options" argument must be an object/;
+const empty = `${common.fixturesDir}/empty.js`;
 
 assert.throws(function() {
-  var child = spawn(invalidcmd, 'this is not an array');
-  child.on('error', common.fail);
+  const child = spawn(invalidcmd, 'this is not an array');
+  child.on('error', common.mustNotCall());
 }, TypeError);
 
 // verify that valid argument combinations do not throw
@@ -57,7 +57,7 @@ assert.throws(function() {
 // Argument types for combinatorics
 const a = [];
 const o = {};
-const c = function c() {};
+function c() {}
 const s = 'string';
 const u = undefined;
 const n = null;
