@@ -1,19 +1,21 @@
 'use strict';
 
+const common = require('../common');
 const assert = require('assert');
 const spawnSync = require('child_process').spawnSync;
 const path = require('path');
 
-const common = require('../common');
-
-var node = process.execPath;
+const node = process.execPath;
 
 // test both sets of arguments that check syntax
-var syntaxArgs = [
+const syntaxArgs = [
   ['-c'],
   ['--check']
 ];
 
+const syntaxErrorRE = /^SyntaxError: Unexpected identifier$/m;
+const notFoundRE = /^Error: Cannot find module/m;
+
 // test good syntax with and without shebang
 [
   'syntax/good_syntax.js',
@@ -26,13 +28,13 @@ var syntaxArgs = [
 
   // loop each possible option, `-c` or `--check`
   syntaxArgs.forEach(function(args) {
-    var _args = args.concat(file);
-    var c = spawnSync(node, _args, {encoding: 'utf8'});
+    const _args = args.concat(file);
+    const c = spawnSync(node, _args, {encoding: 'utf8'});
 
     // no output should be produced
     assert.strictEqual(c.stdout, '', 'stdout produced');
     assert.strictEqual(c.stderr, '', 'stderr produced');
-    assert.strictEqual(c.status, 0, 'code == ' + c.status);
+    assert.strictEqual(c.status, 0, `code == ${c.status}`);
   });
 });
 
@@ -47,17 +49,16 @@ var syntaxArgs = [
 
   // loop each possible option, `-c` or `--check`
   syntaxArgs.forEach(function(args) {
-    var _args = args.concat(file);
-    var c = spawnSync(node, _args, {encoding: 'utf8'});
+    const _args = args.concat(file);
+    const c = spawnSync(node, _args, {encoding: 'utf8'});
 
     // no stdout should be produced
     assert.strictEqual(c.stdout, '', 'stdout produced');
 
     // stderr should have a syntax error message
-    var match = c.stderr.match(/^SyntaxError: Unexpected identifier$/m);
-    assert(match, 'stderr incorrect');
+    assert(syntaxErrorRE.test(c.stderr), 'stderr incorrect');
 
-    assert.strictEqual(c.status, 1, 'code == ' + c.status);
+    assert.strictEqual(c.status, 1, `code == ${c.status}`);
   });
 });
 
@@ -70,16 +71,15 @@ var syntaxArgs = [
 
   // loop each possible option, `-c` or `--check`
   syntaxArgs.forEach(function(args) {
-    var _args = args.concat(file);
-    var c = spawnSync(node, _args, {encoding: 'utf8'});
+    const _args = args.concat(file);
+    const c = spawnSync(node, _args, {encoding: 'utf8'});
 
     // no stdout should be produced
     assert.strictEqual(c.stdout, '', 'stdout produced');
 
     // stderr should have a module not found error message
-    var match = c.stderr.match(/^Error: Cannot find module/m);
-    assert(match, 'stderr incorrect');
+    assert(notFoundRE.test(c.stderr), 'stderr incorrect');
 
-    assert.strictEqual(c.status, 1, 'code == ' + c.status);
+    assert.strictEqual(c.status, 1, `code == ${c.status}`);
   });
 });
