@@ -1,18 +1,17 @@
 'use strict';
-var common = require('../common');
-var domain = require('domain');
-var assert = require('assert');
-var d = domain.create();
-var expect = ['pbkdf2', 'randomBytes', 'pseudoRandomBytes'];
-
-if (!common.hasCrypto) {
+const common = require('../common');
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
-var crypto = require('crypto');
+
+const domain = require('domain');
+const assert = require('assert');
+const crypto = require('crypto');
+
+const d = domain.create();
+const expect = ['pbkdf2', 'randomBytes', 'pseudoRandomBytes'];
 
 d.on('error', common.mustCall(function(e) {
-  assert.equal(e.message, expect.shift());
+  assert.strictEqual(e.message, expect.shift());
 }, 3));
 
 d.run(function() {
