@@ -1,18 +1,18 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
+const common = require('../common');
+const assert = require('assert');
 
-var path = require('path');
-var fs = require('fs');
-var fn = path.join(common.tmpDir, 'write.txt');
+const path = require('path');
+const fs = require('fs');
+const fn = path.join(common.tmpDir, 'write.txt');
 common.refreshTmpDir();
-var file = fs.createWriteStream(fn, {
+const file = fs.createWriteStream(fn, {
   highWaterMark: 10
 });
 
-var EXPECTED = '012345678910';
+const EXPECTED = '012345678910';
 
-var callbacks = {
+const callbacks = {
   open: -1,
   drain: -2,
   close: -1
@@ -22,7 +22,7 @@ file
   .on('open', function(fd) {
     console.error('open!');
     callbacks.open++;
-    assert.equal('number', typeof fd);
+    assert.strictEqual('number', typeof fd);
   })
   .on('error', function(err) {
     throw err;
@@ -30,11 +30,11 @@ file
   .on('drain', function() {
     console.error('drain!', callbacks.drain);
     callbacks.drain++;
-    if (callbacks.drain == -1) {
-      assert.equal(EXPECTED, fs.readFileSync(fn, 'utf8'));
+    if (callbacks.drain === -1) {
+      assert.strictEqual(EXPECTED, fs.readFileSync(fn, 'utf8'));
       file.write(EXPECTED);
-    } else if (callbacks.drain == 0) {
-      assert.equal(EXPECTED + EXPECTED, fs.readFileSync(fn, 'utf8'));
+    } else if (callbacks.drain === 0) {
+      assert.strictEqual(EXPECTED + EXPECTED, fs.readFileSync(fn, 'utf8'));
       file.end();
     }
   })
@@ -51,15 +51,13 @@ file
     fs.unlinkSync(fn);
   });
 
-for (var i = 0; i < 11; i++) {
-  (function(i) {
-    file.write('' + i);
-  })(i);
+for (let i = 0; i < 11; i++) {
+  file.write(`${i}`);
 }
 
 process.on('exit', function() {
-  for (var k in callbacks) {
-    assert.equal(0, callbacks[k], k + ' count off by ' + callbacks[k]);
+  for (const k in callbacks) {
+    assert.strictEqual(0, callbacks[k], `${k} count off by ${callbacks[k]}`);
   }
   console.log('ok');
 });
