@@ -7,29 +7,32 @@ const fs = require('fs');
 const emptyFile = path.join(common.fixturesDir, 'empty.txt');
 
 fs.open(emptyFile, 'r', common.mustCall((error, fd) => {
+
   assert.ifError(error);
 
-  const read = fs.createReadStream(emptyFile, { 'fd': fd });
+  const read = fs.createReadStream(emptyFile, { fd });
 
   read.once('data', () => {
-    throw new Error('data event should not emit');
+    common.fail('data event should not emit');
   });
 
   read.once('end', common.mustCall(function endEvent1() {}));
 }));
 
 fs.open(emptyFile, 'r', common.mustCall((error, fd) => {
+
   assert.ifError(error);
 
-  const read = fs.createReadStream(emptyFile, { 'fd': fd });
+  const read = fs.createReadStream(emptyFile, { fd });
+
   read.pause();
 
   read.once('data', () => {
-    throw new Error('data event should not emit');
+    common.fail('data event should not emit');
   });
 
   read.once('end', function endEvent2() {
-    throw new Error('end event should not emit');
+    common.fail('end event should not emit');
   });
 
   setTimeout(common.mustCall(() => {
