@@ -1,17 +1,17 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var path = require('path');
-var fs = require('fs');
+const common = require('../common');
+const assert = require('assert');
+const path = require('path');
+const fs = require('fs');
 
 // test creating and reading symbolic link
-var linkData = path.join(common.fixturesDir, 'cycles/');
-var linkPath = path.join(common.tmpDir, 'cycles_link');
+const linkData = path.join(common.fixturesDir, 'cycles/');
+const linkPath = path.join(common.tmpDir, 'cycles_link');
 
 common.refreshTmpDir();
 
-console.log('linkData: ' + linkData);
-console.log('linkPath: ' + linkPath);
+console.log(`linkData: ${linkData}`);
+console.log(`linkPath: ${linkPath}`);
 
 fs.symlink(linkData, linkPath, 'junction', common.mustCall(function(err) {
   if (err) throw err;
@@ -22,7 +22,7 @@ fs.symlink(linkData, linkPath, 'junction', common.mustCall(function(err) {
 
     fs.readlink(linkPath, common.mustCall(function(err, destination) {
       if (err) throw err;
-      assert.equal(destination, linkData);
+      assert.strictEqual(destination, linkData);
 
       fs.unlink(linkPath, common.mustCall(function(err) {
         if (err) throw err;
