@@ -1,18 +1,18 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var net = require('net');
-var http = require('http');
-var url = require('url');
+const assert = require('assert');
+const net = require('net');
+const http = require('http');
+const url = require('url');
 
 // Make sure no exceptions are thrown when receiving malformed HTTP
 // requests.
 
-var nrequests_completed = 0;
-var nrequests_expected = 1;
+let nrequests_completed = 0;
+const nrequests_expected = 1;
 
-var server = http.createServer(function(req, res) {
-  console.log('req: ' + JSON.stringify(url.parse(req.url)));
+const server = http.createServer(function(req, res) {
+  console.log(`req: ${JSON.stringify(url.parse(req.url))}`);
 
   res.writeHead(200, {'Content-Type': 'text/plain'});
   res.write('Hello World');
@@ -23,7 +23,7 @@ var server = http.createServer(function(req, res) {
 server.listen(0);
 
 server.on('listening', function() {
-  var c = net.createConnection(this.address().port);
+  const c = net.createConnection(this.address().port);
   c.on('connect', function() {
     c.write('GET /hello?foo=%99bar HTTP/1.1\r\n\r\n');
     c.end();
@@ -31,5 +31,5 @@ server.on('listening', function() {
 });
 
 process.on('exit', function() {
-  assert.equal(nrequests_expected, nrequests_completed);
+  assert.strictEqual(nrequests_expected, nrequests_completed);
 });
