@@ -1,7 +1,7 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var http = require('http');
+const assert = require('assert');
+const http = require('http');
 
 // Simple test of Node's HTTP Client mutable headers
 // OutgoingMessage.prototype.setHeader(name, value)
@@ -11,43 +11,51 @@ var http = require('http');
 // <ClientRequest>.method
 // <ClientRequest>.path
 
-var testsComplete = 0;
-var test = 'headers';
-var content = 'hello world\n';
-var cookies = [
+let testsComplete = 0;
+let test = 'headers';
+const content = 'hello world\n';
+const cookies = [
   'session_token=; path=/; expires=Sun, 15-Sep-2030 13:48:52 GMT',
   'prefers_open_id=; path=/; expires=Thu, 01-Jan-1970 00:00:00 GMT'
 ];
 
-var s = http.createServer(function(req, res) {
+const s = http.createServer(function(req, res) {
   switch (test) {
     case 'headers':
-      assert.throws(function() { res.setHeader(); });
-      assert.throws(function() { res.setHeader('someHeader'); });
-      assert.throws(function() { res.getHeader(); });
-      assert.throws(function() { res.removeHeader(); });
+      assert.throws(() => {
+        res.setHeader();
+      }, /^TypeError: Header name must be a valid HTTP Token \["undefined"\]$/);
+      assert.throws(() => {
+        res.setHeader('someHeader');
+      }, /^Error: "value" required in setHeader\("someHeader", value\)$/);
+      assert.throws(() => {
+        res.getHeader();
+      }, /^Error: "name" argument is required for getHeader\(name\)$/);
+      assert.throws(() => {
+        res.removeHeader();
+      }, /^Error: "name" argument is required for removeHeader\(name\)$/);
 
       res.setHeader('x-test-header', 'testing');
       res.setHeader('X-TEST-HEADER2', 'testing');
       res.setHeader('set-cookie', cookies);
       res.setHeader('x-test-array-header', [1, 2, 3]);
 
-      var val1 = res.getHeader('x-test-header');
-      var val2 = res.getHeader('x-test-header2');
-      assert.equal(val1, 'testing');
-      assert.equal(val2, 'testing');
+      const val1 = res.getHeader('x-test-header');
+      const val2 = res.getHeader('x-test-header2');
+      assert.strictEqual(val1, 'testing');
+      assert.strictEqual(val2, 'testing');
 
       res.removeHeader('x-test-header2');
       break;
 
     case 'contentLength':
       res.setHeader('content-length', content.length);
-      assert.equal(content.length, res.getHeader('Content-Length'));
+      assert.strictEqual(content.length, res.getHeader('Content-Length'));
       break;
 
     case 'transferEncoding':
       res.setHeader('transfer-encoding', 'chunked');
-      assert.equal(res.getHeader('Transfer-Encoding'), 'chunked');
+      assert.strictEqual(res.getHeader('Transfer-Encoding'), 'chunked');
       break;
 
     case 'writeHead':
@@ -69,7 +77,7 @@ function nextTest() {
     return s.close();
   }
 
-  var bufferedResponse = '';
+  let bufferedResponse = '';
 
   http.get({ port: s.address().port }, function(response) {
     console.log('TEST: ' + test);
@@ -79,33 +87,35 @@ function nextTest() {
 
     switch (test) {
       case 'headers':
-        assert.equal(response.statusCode, 201);
-        assert.equal(response.headers['x-test-header'],
-                     'testing');
-        assert.equal(response.headers['x-test-array-header'],
-                     [1, 2, 3].join(', '));
-        assert.deepEqual(cookies,
-                         response.headers['set-cookie']);
-        assert.equal(response.headers['x-test-header2'] !== undefined, false);
+        assert.strictEqual(response.statusCode, 201);
+        assert.strictEqual(response.headers['x-test-header'],
+                           'testing');
+        assert.strictEqual(response.headers['x-test-array-header'],
+                           [1, 2, 3].join(', '));
+        assert.deepStrictEqual(cookies,
+                               response.headers['set-cookie']);
+        assert.strictEqual(response.headers['x-test-header2'] !== undefined,
+                           false);
         // Make the next request
         test = 'contentLength';
         console.log('foobar');
         break;
 
       case 'contentLength':
-        assert.equal(response.headers['content-length'], content.length);
+        assert.strictEqual(response.headers['content-length'],
+                           content.length.toString());
         test = 'transferEncoding';
         break;
 
       case 'transferEncoding':
-        assert.equal(response.headers['transfer-encoding'], 'chunked');
+        assert.strictEqual(response.headers['transfer-encoding'], 'chunked');
         test = 'writeHead';
         break;
 
       case 'writeHead':
-        assert.equal(response.headers['x-foo'], 'bar');
-        assert.equal(response.headers['x-bar'], 'baz');
-        assert.equal(200, response.statusCode);
+        assert.strictEqual(response.headers['x-foo'], 'bar');
+        assert.strictEqual(response.headers['x-bar'], 'baz');
+        assert.strictEqual(200, response.statusCode);
         test = 'end';
         break;
 
@@ -119,7 +129,7 @@ function nextTest() {
     });
 
     response.on('end', function() {
-      assert.equal(content, bufferedResponse);
+      assert.strictEqual(content, bufferedResponse);
       testsComplete++;
       nextTest();
     });
@@ -128,6 +138,5 @@ function nextTest() {
 
 
 process.on('exit', function() {
-  assert.equal(4, testsComplete);
+  assert.strictEqual(4, testsComplete);
 });
-
