@@ -1,29 +1,26 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var http = require('http');
-var fs = require('fs');
-var path = require('path');
+const common = require('../common');
+const http = require('http');
+const fs = require('fs');
+const path = require('path');
 
 common.refreshTmpDir();
 
-var file = path.join(common.tmpDir, 'http-pipe-fs-test.txt');
-var requests = 0;
+const file = path.join(common.tmpDir, 'http-pipe-fs-test.txt');
 
-var server = http.createServer(function(req, res) {
-  ++requests;
-  var stream = fs.createWriteStream(file);
+const server = http.createServer(common.mustCall(function(req, res) {
+  const stream = fs.createWriteStream(file);
   req.pipe(stream);
   stream.on('close', function() {
     res.writeHead(200);
     res.end();
   });
-}).listen(0, function() {
+}, 2)).listen(0, function() {
   http.globalAgent.maxSockets = 1;
 
-  for (var i = 0; i < 2; ++i) {
+  for (let i = 0; i < 2; ++i) {
     (function(i) {
-      var req = http.request({
+      const req = http.request({
         port: server.address().port,
         method: 'POST',
         headers: {
@@ -31,7 +28,7 @@ var server = http.createServer(function(req, res) {
         }
       }, function(res) {
         res.on('end', function() {
-          console.error('res' + i + ' end');
+          console.error(`res${i} end`);
           if (i === 2) {
             server.close();
           }
@@ -39,13 +36,9 @@ var server = http.createServer(function(req, res) {
         res.resume();
       });
       req.on('socket', function(s) {
-        console.error('req' + i + ' start');
+        console.error(`req${i} start`);
       });
       req.end('12345');
     }(i + 1));
   }
 });
-
-process.on('exit', function() {
-  assert.equal(requests, 2);
-});
