@@ -1,10 +1,10 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var http = require('http');
-var url = require('url');
+const assert = require('assert');
+const http = require('http');
+const url = require('url');
 
-var testURL;
+let testURL;
 
 function check(request) {
   //url.parse should not mess with the method
@@ -13,10 +13,10 @@ function check(request) {
   assert.strictEqual(request.url, '/asdf?qwer=zxcv');
   //the host header should use the url.parse.hostname
   assert.strictEqual(request.headers.host,
-                     testURL.hostname + ':' + testURL.port);
+                     `${testURL.hostname}:${testURL.port}`);
 }
 
-var server = http.createServer(function(request, response) {
+const server = http.createServer(function(request, response) {
   // run the check function
   check.call(this, request, response);
   response.writeHead(200, {});
