@@ -1,14 +1,12 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var http = require('http');
-var net = require('net');
-var spawn = require('child_process').spawn;
-
-if (common.isWindows) {
+const common = require('../common');
+if (common.isWindows)
   common.skip('This test is disabled on windows.');
-  return;
-}
+
+const assert = require('assert');
+const http = require('http');
+const net = require('net');
+const spawn = require('child_process').spawn;
 
 switch (process.argv[2]) {
   case 'child': return child();
@@ -23,24 +21,24 @@ switch (process.argv[2]) {
 // concurrency in HTTP servers!  Use the cluster module, or if you want
 // a more low-level approach, use child process IPC manually.
 function test() {
-  var parent = spawn(process.execPath, [__filename, 'parent'], {
+  const parent = spawn(process.execPath, [__filename, 'parent'], {
     stdio: [ 0, 'pipe', 2 ]
   });
-  var json = '';
+  let json = '';
   parent.stdout.on('data', function(c) {
     json += c.toString();
-    if (json.indexOf('\n') !== -1) next();
+    if (json.includes('\n')) next();
   });
   function next() {
     console.error('output from parent = %s', json);
-    var child = JSON.parse(json);
+    const child = JSON.parse(json);
     // now make sure that we can request to the child, then kill it.
     http.get({
       server: 'localhost',
       port: child.port,
       path: '/',
     }).on('response', function(res) {
-      var s = '';
+      let s = '';
       res.on('data', function(c) {
         s += c.toString();
       });
@@ -52,22 +50,21 @@ function test() {
           parent.kill();
         } catch (e) {}
 
-        assert.equal(s, 'hello from child\n');
-        assert.equal(res.statusCode, 200);
+        assert.strictEqual(s, 'hello from child\n');
+        assert.strictEqual(res.statusCode, 200);
       });
     });
   }
 }
 
 function parent() {
-  var server = net.createServer(function(conn) {
+  const server = net.createServer(function(conn) {
     console.error('connection on parent');
     conn.end('hello from parent\n');
   }).listen(0, function() {
     console.error('server listening on %d', this.address().port);
 
-    var spawn = require('child_process').spawn;
-    var child = spawn(process.execPath, [__filename, 'child'], {
+    const child = spawn(process.execPath, [__filename, 'child'], {
       stdio: [ 'ignore', 'ignore', 'ignore', server._handle ],
       detached: true
     });
@@ -93,4 +90,3 @@ function child() {
     console.error('child listening on fd=3');
   });
 }
-
