@@ -1,6 +1,6 @@
 'use strict';
 require('../common');
-var assert = require('assert');
+const assert = require('assert');
 
 // Requiring the domain module here changes the function that is used by node to
 // call process.nextTick's callbacks to a variant that specifically handles
@@ -13,10 +13,10 @@ function enqueueMicrotask(fn) {
   Promise.resolve().then(fn);
 }
 
-var done = 0;
+let done = 0;
 
 process.on('exit', function() {
-  assert.equal(done, 2);
+  assert.strictEqual(done, 2);
 });
 
 // no nextTick, microtask
@@ -29,7 +29,7 @@ setImmediate(function() {
 
 // no nextTick, microtask with nextTick
 setImmediate(function() {
-  var called = false;
+  let called = false;
 
   enqueueMicrotask(function() {
     process.nextTick(function() {
