@@ -1,21 +1,21 @@
 'use strict';
 const common = require('../common');
-var assert = require('assert');
+const assert = require('assert');
 
 console.error('load test-module-loading-error.js');
 
-var error_desc = {
+const error_desc = {
   win32: ['%1 is not a valid Win32 application'],
   linux: ['file too short', 'Exec format error'],
   sunos: ['unknown file type', 'not an ELF file'],
-  darwin: ['file too short']
+  darwin: ['file too short'],
+  aix: ['Cannot load module',
+        'Cannot run a file that does not have a valid format.']
 };
-var dlerror_msg = error_desc[process.platform];
+const dlerror_msg = error_desc[process.platform];
 
-if (!dlerror_msg) {
+if (!dlerror_msg)
   common.skip('platform not supported.');
-  return;
-}
 
 try {
   require('../fixtures/module-loading-error.node');
@@ -28,11 +28,11 @@ try {
 try {
   require();
 } catch (e) {
-  assert.notEqual(e.toString().indexOf('missing path'), -1);
+  assert.notStrictEqual(e.toString().indexOf('missing path'), -1);
 }
 
 try {
   require({});
 } catch (e) {
-  assert.notEqual(e.toString().indexOf('path must be a string'), -1);
+  assert.notStrictEqual(e.toString().indexOf('path must be a string'), -1);
 }
