@@ -1,11 +1,11 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var net = require('net');
+const assert = require('assert');
+const net = require('net');
 
-var iter = 10;
+const iter = 10;
 
-var server = net.createServer(function(socket) {
+const server = net.createServer(function(socket) {
   socket.on('readable', function() {
     socket.read();
   });
@@ -16,13 +16,13 @@ var server = net.createServer(function(socket) {
 });
 
 server.listen(0, function() {
-  var client = net.connect(this.address().port);
+  const client = net.connect(this.address().port);
 
   client.on('finish', function() {
     assert.strictEqual(client.bufferSize, 0);
   });
 
-  for (var i = 1; i < iter; i++) {
+  for (let i = 1; i < iter; i++) {
     client.write('a');
     assert.strictEqual(client.bufferSize, i);
   }
