@@ -1,15 +1,15 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var net = require('net');
+const assert = require('assert');
+const net = require('net');
 
 // Server should only fire listen callback once
-var server = net.createServer();
+const server = net.createServer();
 
 server.on('close', function() {
-  var listeners = server.listeners('listening');
+  const listeners = server.listeners('listening');
   console.log('Closed, listeners:', listeners.length);
-  assert.equal(0, listeners.length);
+  assert.strictEqual(0, listeners.length);
 });
 
 server.listen(0, function() {
