@@ -7,7 +7,7 @@ const net = require('net');
 const server = net.createServer(handle);
 
 const N = 100;
-const buf = Buffer('aa');
+const buf = Buffer.alloc(2, 'a');
 
 server.listen(0, function() {
   const conn = net.connect(this.address().port);
@@ -21,13 +21,13 @@ server.listen(0, function() {
       res = conn.write(buf);
       conn.uncork();
     }
-    assert.equal(i, N);
+    assert.strictEqual(i, N);
     conn.end();
   });
 });
 
 process.on('exit', function() {
-  assert.equal(server.connections, 0);
+  assert.strictEqual(server.connections, 0);
 });
 
 function handle(socket) {
