@@ -1,23 +1,22 @@
 'use strict';
 const common = require('../common');
-var assert = require('assert');
-var net = require('net');
+const assert = require('assert');
+const net = require('net');
 
-var SIZE = 2E5;
-var N = 10;
-var flushed = 0;
-var received = 0;
-var buf = new Buffer(SIZE);
-buf.fill(0x61); // 'a'
+const SIZE = 2E5;
+const N = 10;
+let flushed = 0;
+let received = 0;
+const buf = Buffer.alloc(SIZE, 'a');
 
-var server = net.createServer(function(socket) {
+const server = net.createServer(function(socket) {
   socket.setNoDelay();
   socket.setTimeout(9999);
   socket.on('timeout', function() {
     common.fail(`flushed: ${flushed}, received: ${received}/${SIZE * N}`);
   });
 
-  for (var i = 0; i < N; ++i) {
+  for (let i = 0; i < N; ++i) {
     socket.write(buf, function() {
       ++flushed;
       if (flushed === N) {
@@ -28,7 +27,7 @@ var server = net.createServer(function(socket) {
   socket.end();
 
 }).listen(0, common.mustCall(function() {
-  var conn = net.connect(this.address().port);
+  const conn = net.connect(this.address().port);
   conn.on('data', function(buf) {
     received += buf.length;
     conn.pause();
