@@ -1,7 +1,7 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var os = require('os');
+const assert = require('assert');
+const os = require('os');
 
 switch (process.argv[2]) {
   case 'child':
@@ -9,14 +9,14 @@ switch (process.argv[2]) {
   case undefined:
     return parent();
   default:
-    throw new Error('wtf? ' + process.argv[2]);
+    throw new Error(`wtf? ${process.argv[2]}`);
 }
 
 function parent() {
-  var spawn = require('child_process').spawn;
-  var child = spawn(process.execPath, [__filename, 'child']);
+  const spawn = require('child_process').spawn;
+  const child = spawn(process.execPath, [__filename, 'child']);
 
-  var output = '';
+  let output = '';
 
   child.stderr.on('data', function(c) {
     output += c;
@@ -25,7 +25,7 @@ function parent() {
   child.stderr.setEncoding('utf8');
 
   child.stderr.on('end', function() {
-    assert.equal(output, 'I can still debug!' + os.EOL);
+    assert.strictEqual(output, `I can still debug!${os.EOL}`);
     console.log('ok - got expected message');
   });
 
@@ -42,7 +42,7 @@ function child() {
     throw new Error('No ticking!');
   };
 
-  var stderr = process.stderr;
+  const stderr = process.stderr;
   stderr.write = function() {
     throw new Error('No writing to stderr!');
   };
