@@ -2,7 +2,7 @@
 
 // Flags: --expose-internals
 
-require('../common');
+const common = require('../common');
 const stream = require('stream');
 const REPL = require('internal/repl');
 const assert = require('assert');
@@ -36,7 +36,7 @@ const tests = [
 ];
 
 function run(test) {
-  const env = test.env;
+  const env = Object.assign({}, process.env, test.env);
   const expected = test.expected;
   const opts = {
     terminal: true,
@@ -46,10 +46,14 @@ function run(test) {
 
   REPL.createInternalRepl(env, opts, function(err, repl) {
     if (err) throw err;
-    assert.equal(expected.terminal, repl.terminal,
-                 'Expected ' + inspect(expected) + ' with ' + inspect(env));
-    assert.equal(expected.useColors, repl.useColors,
-                 'Expected ' + inspect(expected) + ' with ' + inspect(env));
+
+    // The REPL registers 'module' and 'require' globals
+    common.allowGlobals(repl.context.module, repl.context.require);
+
+    assert.strictEqual(expected.terminal, repl.terminal,
+                       `Expected ${inspect(expected)} with ${inspect(env)}`);
+    assert.strictEqual(expected.useColors, repl.useColors,
+                       `Expected ${inspect(expected)} with ${inspect(env)}`);
     repl.close();
   });
 }
