@@ -1,4 +1,5 @@
-/* eslint-disable max-len, strict */
+'use strict';
+
 const common = require('../common');
 const assert = require('assert');
 
@@ -15,7 +16,7 @@ const prompt_npm = 'npm should be run outside of the ' +
                    'node repl, in your normal shell.\n' +
                    '(Press Control-D to exit.)\n';
 const expect_npm = prompt_npm + prompt_unix;
-var server_tcp, server_unix, client_tcp, client_unix, replServer;
+let server_tcp, server_unix, client_tcp, client_unix, replServer;
 
 
 // absolute path to test/fixtures/a.js
@@ -25,19 +26,19 @@ console.error('repl test');
 
 // function for REPL to run
 global.invoke_me = function(arg) {
-  return 'invoked ' + arg;
+  return `invoked ${arg}`;
 };
 
 function send_expect(list) {
   if (list.length > 0) {
     const cur = list.shift();
 
-    console.error('sending ' + JSON.stringify(cur.send));
+    console.error(`sending ${JSON.stringify(cur.send)}`);
 
     cur.client.expect = cur.expect;
     cur.client.list = list;
     if (cur.send.length > 0) {
-      cur.client.write(cur.send + '\n');
+      cur.client.write(`${cur.send}\n`);
     }
   }
 }
@@ -56,24 +57,25 @@ function strict_mode_error_test() {
 
 function error_test() {
   // The other stuff is done so reuse unix socket
-  var read_buffer = '';
-  var run_strict_test = true;
+  let read_buffer = '';
+  let run_strict_test = true;
   client_unix.removeAllListeners('data');
 
   client_unix.on('data', function(data) {
     read_buffer += data.toString('ascii', 0, data.length);
-    console.error('Unix data: ' + JSON.stringify(read_buffer) + ', expecting ' +
-                 (client_unix.expect.exec ?
-                  client_unix.expect :
-                  JSON.stringify(client_unix.expect)));
+    console.error(
+      `Unix data: ${JSON.stringify(read_buffer)}, expecting ${
+        client_unix.expect.exec ?
+          client_unix.expect :
+          JSON.stringify(client_unix.expect)}`);
 
-    if (read_buffer.indexOf(prompt_unix) !== -1) {
+    if (read_buffer.includes(prompt_unix)) {
       // if it's an exact match, then don't do the regexp
       if (read_buffer !== client_unix.expect) {
-        var expect = client_unix.expect;
+        let expect = client_unix.expect;
         if (expect === prompt_multiline)
           expect = /[.]{3} /;
-        assert.ok(read_buffer.match(expect));
+        assert.ok(RegExp(expect).test(read_buffer));
         console.error('match');
       }
       read_buffer = '';
@@ -88,7 +90,7 @@ function error_test() {
         tcp_test();
       }
 
-    } else if (read_buffer.indexOf(prompt_multiline) !== -1) {
+    } else if (read_buffer.includes(prompt_multiline)) {
       // Check that you meant to send a multiline test
       assert.strictEqual(prompt_multiline, client_unix.expect);
       read_buffer = '';
@@ -120,7 +122,7 @@ function error_test() {
       expect: prompt_unix },
     // But passing the same string to eval() should throw
     { client: client_unix, send: 'eval("function test_func() {")',
-      expect: /^SyntaxError: Unexpected end of input/ },
+      expect: /\bSyntaxError: Unexpected end of input/ },
     // Can handle multiline template literals
     { client: client_unix, send: '`io.js',
       expect: prompt_multiline },
@@ -132,6 +134,11 @@ function error_test() {
       expect: prompt_multiline },
     { client: client_unix, send: '+ ".2"}`',
       expect: `'io.js 1.0.2'\n${prompt_unix}` },
+    // Dot prefix in multiline commands aren't treated as commands
+    { client: client_unix, send: '("a"',
+      expect: prompt_multiline },
+    { client: client_unix, send: '.charAt(0))',
+      expect: `'a'\n${prompt_unix}` },
     // Floating point numbers are not interpreted as REPL commands.
     { client: client_unix, send: '.1234',
       expect: '0.1234' },
@@ -144,42 +151,56 @@ function error_test() {
     // invalid input to JSON.parse error is special case of syntax error,
     // should throw
     { client: client_unix, send: 'JSON.parse(\'{invalid: \\\'json\\\'}\');',
-      expect: /^SyntaxError: Unexpected token i/ },
+      expect: /\bSyntaxError: Unexpected token i/ },
     // end of input to JSON.parse error is special case of syntax error,
     // should throw
     { client: client_unix, send: 'JSON.parse(\'066\');',
-      expect: /^SyntaxError: Unexpected number/ },
+      expect: /\bSyntaxError: Unexpected number/ },
     // should throw
     { client: client_unix, send: 'JSON.parse(\'{\');',
-      expect: /^SyntaxError: Unexpected end of input/ },
+      expect: /\bSyntaxError: Unexpected end of JSON input/ },
     // invalid RegExps are a special case of syntax error,
     // should throw
     { client: client_unix, send: '/(/;',
-      expect: /^SyntaxError: Invalid regular expression\:/ },
+      expect: /\bSyntaxError: Invalid regular expression:/ },
     // invalid RegExp modifiers are a special case of syntax error,
     // should throw (GH-4012)
     { client: client_unix, send: 'new RegExp("foo", "wrong modifier");',
-      expect: /^SyntaxError: Invalid flags supplied to RegExp constructor/ },
+      expect: /\bSyntaxError: Invalid flags supplied to RegExp constructor/ },
     // strict mode syntax errors should be caught (GH-5178)
-    { client: client_unix, send: '(function() { "use strict"; return 0755; })()',
-      expect: /^SyntaxError: Octal literals are not allowed in strict mode/ },
-    { client: client_unix, send: '(function(a, a, b) { "use strict"; return a + b + c; })()',
-      expect: /^SyntaxError: Strict mode function may not have duplicate parameter names/ },
-    { client: client_unix, send: '(function() { "use strict"; with (this) {} })()',
-      expect: /^SyntaxError: Strict mode code may not include a with statement/ },
-    { client: client_unix, send: '(function() { "use strict"; var x; delete x; })()',
-      expect: /^SyntaxError: Delete of an unqualified identifier in strict mode/ },
-    { client: client_unix, send: '(function() { "use strict"; eval = 17; })()',
-      expect: /^SyntaxError: Unexpected eval or arguments in strict mode/ },
-    { client: client_unix, send: '(function() { "use strict"; if (true) function f() { } })()',
-      expect: /^SyntaxError: In strict mode code, functions can only be declared at top level or immediately within another function/ },
+    { client: client_unix,
+      send: '(function() { "use strict"; return 0755; })()',
+      expect: /\bSyntaxError: Octal literals are not allowed in strict mode/ },
+    {
+      client: client_unix,
+      send: '(function(a, a, b) { "use strict"; return a + b + c; })()',
+      expect: /\bSyntaxError: Duplicate parameter name not allowed in this context/ // eslint-disable-line max-len
+    },
+    {
+      client: client_unix,
+      send: '(function() { "use strict"; with (this) {} })()',
+      expect: /\bSyntaxError: Strict mode code may not include a with statement/
+    },
+    {
+      client: client_unix,
+      send: '(function() { "use strict"; var x; delete x; })()',
+      expect: /\bSyntaxError: Delete of an unqualified identifier in strict mode/ // eslint-disable-line max-len
+    },
+    { client: client_unix,
+      send: '(function() { "use strict"; eval = 17; })()',
+      expect: /\bSyntaxError: Unexpected eval or arguments in strict mode/ },
+    {
+      client: client_unix,
+      send: '(function() { "use strict"; if (true) function f() { } })()',
+      expect: /\bSyntaxError: In strict mode code, functions can only be declared at top level or inside a block./ // eslint-disable-line max-len
+    },
     // Named functions can be used:
     { client: client_unix, send: 'function blah() { return 1; }',
       expect: prompt_unix },
     { client: client_unix, send: 'blah()',
-      expect: '1\n' + prompt_unix },
+      expect: `1\n${prompt_unix}` },
     // Functions should not evaluate twice (#2773)
-    { client: client_unix, send: 'var I = [1,2,3,function() {}]; I.pop()',
+    { client: client_unix, send: 'var I = [1,2,3,() => {}]; I.pop()',
       expect: '[Function]' },
     // Multiline object
     { client: client_unix, send: '{ a: ',
@@ -195,6 +216,13 @@ function error_test() {
       expect: prompt_multiline },
     { client: client_unix, send: '})()',
       expect: '1' },
+    // Multiline function call
+    { client: client_unix, send: 'function f(){}; f(f(1,',
+      expect: prompt_multiline },
+    { client: client_unix, send: '2)',
+      expect: prompt_multiline },
+    { client: client_unix, send: ')',
+      expect: `undefined\n${prompt_unix}` },
     // npm prompt error message
     { client: client_unix, send: 'npm install foobar',
       expect: expect_npm },
@@ -209,18 +237,18 @@ function error_test() {
     // this makes sure that we don't print `undefined` when we actually print
     // the error message
     { client: client_unix, send: '.invalid_repl_command',
-      expect: 'Invalid REPL keyword\n' + prompt_unix },
+      expect: `Invalid REPL keyword\n${prompt_unix}` },
     // this makes sure that we don't crash when we use an inherited property as
     // a REPL command
     { client: client_unix, send: '.toString',
-      expect: 'Invalid REPL keyword\n' + prompt_unix },
+      expect: `Invalid REPL keyword\n${prompt_unix}` },
     // fail when we are not inside a String and a line continuation is used
     { client: client_unix, send: '[] \\',
-      expect: /^SyntaxError: Unexpected token ILLEGAL/ },
+      expect: /\bSyntaxError: Invalid or unexpected token/ },
     // do not fail when a String is created with line continuation
     { client: client_unix, send: '\'the\\\nfourth\\\neye\'',
-      expect: prompt_multiline + prompt_multiline +
-              '\'thefourtheye\'\n' + prompt_unix },
+      expect: `${prompt_multiline}${prompt_multiline}'thefourtheye'\n${
+        prompt_unix}` },
     // Don't fail when a partial String is created and line continuation is used
     // with whitespace characters at the end of the string. We are to ignore it.
     // This test is to make sure that we properly remove the whitespace
@@ -229,12 +257,12 @@ function error_test() {
       expect: prompt_unix },
     // multiline strings preserve whitespace characters in them
     { client: client_unix, send: '\'the \\\n   fourth\t\t\\\n  eye  \'',
-      expect: prompt_multiline + prompt_multiline +
-              '\'the    fourth\\t\\t  eye  \'\n' + prompt_unix },
+      expect: `${prompt_multiline}${
+        prompt_multiline}'the    fourth\\t\\t  eye  '\n${prompt_unix}` },
     // more than one multiline strings also should preserve whitespace chars
     { client: client_unix, send: '\'the \\\n   fourth\' +  \'\t\t\\\n  eye  \'',
-      expect: prompt_multiline + prompt_multiline +
-              '\'the    fourth\\t\\t  eye  \'\n' + prompt_unix },
+      expect: `${prompt_multiline}${
+        prompt_multiline}'the    fourth\\t\\t  eye  '\n${prompt_unix}` },
     // using REPL commands within a string literal should still work
     { client: client_unix, send: '\'\\\n.break',
       expect: prompt_unix },
@@ -246,8 +274,8 @@ function error_test() {
       expect: prompt_unix + prompt_unix + prompt_unix },
     // empty lines in the string literals should not affect the string
     { client: client_unix, send: '\'the\\\n\\\nfourtheye\'\n',
-      expect: prompt_multiline + prompt_multiline +
-              '\'thefourtheye\'\n' + prompt_unix },
+      expect: `${prompt_multiline}${
+        prompt_multiline}'thefourtheye'\n${prompt_unix}` },
     // Regression test for https://github.com/nodejs/node/issues/597
     { client: client_unix,
       send: '/(.)(.)(.)(.)(.)(.)(.)(.)(.)/.test(\'123456789\')\n',
@@ -260,56 +288,112 @@ function error_test() {
                '\'7\'\n', '\'8\'\n', '\'9\'\n'].join(`${prompt_unix}`) },
     // regression tests for https://github.com/nodejs/node/issues/2749
     { client: client_unix, send: 'function x() {\nreturn \'\\n\';\n }',
-      expect: prompt_multiline + prompt_multiline +
-              'undefined\n' + prompt_unix },
+      expect: `${prompt_multiline}${prompt_multiline}undefined\n${
+        prompt_unix}` },
     { client: client_unix, send: 'function x() {\nreturn \'\\\\\';\n }',
-      expect: prompt_multiline + prompt_multiline +
-              'undefined\n' + prompt_unix },
+      expect: `${prompt_multiline}${prompt_multiline}undefined\n${
+        prompt_unix}` },
     // regression tests for https://github.com/nodejs/node/issues/3421
     { client: client_unix, send: 'function x() {\n//\'\n }',
-      expect: prompt_multiline + prompt_multiline +
-              'undefined\n' + prompt_unix },
+      expect: `${prompt_multiline}${prompt_multiline}undefined\n${
+        prompt_unix}` },
     { client: client_unix, send: 'function x() {\n//"\n }',
-      expect: prompt_multiline + prompt_multiline +
-              'undefined\n' + prompt_unix },
+      expect: `${prompt_multiline}${prompt_multiline}undefined\n${
+        prompt_unix}` },
     { client: client_unix, send: 'function x() {//\'\n }',
-      expect: prompt_multiline + 'undefined\n' + prompt_unix },
+      expect: `${prompt_multiline}undefined\n${prompt_unix}` },
     { client: client_unix, send: 'function x() {//"\n }',
-      expect: prompt_multiline + 'undefined\n' + prompt_unix },
+      expect: `${prompt_multiline}undefined\n${prompt_unix}` },
     { client: client_unix, send: 'function x() {\nvar i = "\'";\n }',
-      expect: prompt_multiline + prompt_multiline +
-              'undefined\n' + prompt_unix },
+      expect: `${prompt_multiline}${prompt_multiline}undefined\n${
+        prompt_unix}` },
     { client: client_unix, send: 'function x(/*optional*/) {}',
-      expect: 'undefined\n' + prompt_unix },
+      expect: `undefined\n${prompt_unix}` },
     { client: client_unix, send: 'function x(/* // 5 */) {}',
-      expect: 'undefined\n' + prompt_unix },
+      expect: `undefined\n${prompt_unix}` },
     { client: client_unix, send: '// /* 5 */',
-      expect: 'undefined\n' + prompt_unix },
+      expect: `undefined\n${prompt_unix}` },
     { client: client_unix, send: '"//"',
-      expect: '\'//\'\n' + prompt_unix },
+      expect: `'//'\n${prompt_unix}` },
     { client: client_unix, send: '"data /*with*/ comment"',
-      expect: '\'data /*with*/ comment\'\n' + prompt_unix },
+      expect: `'data /*with*/ comment'\n${prompt_unix}` },
     { client: client_unix, send: 'function x(/*fn\'s optional params*/) {}',
-      expect: 'undefined\n' + prompt_unix },
+      expect: `undefined\n${prompt_unix}` },
     { client: client_unix, send: '/* \'\n"\n\'"\'\n*/',
-      expect: 'undefined\n' + prompt_unix },
+      expect: `undefined\n${prompt_unix}` },
     // REPL should get a normal require() function, not one that allows
     // access to internal modules without the --expose_internals flag.
     { client: client_unix, send: 'require("internal/repl")',
       expect: /^Error: Cannot find module 'internal\/repl'/ },
     // REPL should handle quotes within regexp literal in multiline mode
-    { client: client_unix, send: "function x(s) {\nreturn s.replace(/'/,'');\n}",
-      expect: prompt_multiline + prompt_multiline +
-            'undefined\n' + prompt_unix },
-    { client: client_unix, send: "function x(s) {\nreturn s.replace(/\'/,'');\n}",
-      expect: prompt_multiline + prompt_multiline +
-            'undefined\n' + prompt_unix },
-    { client: client_unix, send: 'function x(s) {\nreturn s.replace(/"/,"");\n}',
-      expect: prompt_multiline + prompt_multiline +
-            'undefined\n' + prompt_unix },
-    { client: client_unix, send: 'function x(s) {\nreturn s.replace(/.*/,"");\n}',
-      expect: prompt_multiline + prompt_multiline +
-            'undefined\n' + prompt_unix },
+    { client: client_unix,
+      send: "function x(s) {\nreturn s.replace(/'/,'');\n}",
+      expect: `${prompt_multiline}${prompt_multiline}` +
+            `undefined\n${prompt_unix}` },
+    { client: client_unix,
+      send: "function x(s) {\nreturn s.replace(/'/,'');\n}",
+      expect: `${prompt_multiline}${prompt_multiline}` +
+            `undefined\n${prompt_unix}` },
+    { client: client_unix,
+      send: 'function x(s) {\nreturn s.replace(/"/,"");\n}',
+      expect: `${prompt_multiline}${prompt_multiline}` +
+            `undefined\n${prompt_unix}` },
+    { client: client_unix,
+      send: 'function x(s) {\nreturn s.replace(/.*/,"");\n}',
+      expect: `${prompt_multiline}${prompt_multiline}` +
+            `undefined\n${prompt_unix}` },
+    { client: client_unix, send: '{ var x = 4; }',
+      expect: `undefined\n${prompt_unix}` },
+    // Illegal token is not recoverable outside string literal, RegExp literal,
+    // or block comment. https://github.com/nodejs/node/issues/3611
+    { client: client_unix, send: 'a = 3.5e',
+      expect: /\bSyntaxError: Invalid or unexpected token/ },
+    // Mitigate https://github.com/nodejs/node/issues/548
+    { client: client_unix, send: 'function name(){ return "node"; };name()',
+      expect: `'node'\n${prompt_unix}` },
+    { client: client_unix, send: 'function name(){ return "nodejs"; };name()',
+      expect: `'nodejs'\n${prompt_unix}` },
+    // Avoid emitting repl:line-number for SyntaxError
+    { client: client_unix, send: 'a = 3.5e',
+      expect: /^(?!repl)/ },
+    // Avoid emitting stack trace
+    { client: client_unix, send: 'a = 3.5e',
+      expect: /^(?!\s+at\s)/m },
+
+    // https://github.com/nodejs/node/issues/9850
+    { client: client_unix, send: 'function* foo() {}; foo().next();',
+      expect: '{ value: undefined, done: true }' },
+
+    { client: client_unix, send: 'function *foo() {}; foo().next();',
+      expect: '{ value: undefined, done: true }' },
+
+    { client: client_unix, send: 'function*foo() {}; foo().next();',
+      expect: '{ value: undefined, done: true }' },
+
+    { client: client_unix, send: 'function * foo() {}; foo().next()',
+      expect: '{ value: undefined, done: true }' },
+
+    // https://github.com/nodejs/node/issues/9300
+    {
+      client: client_unix, send: 'function foo() {\nvar bar = 1 / 1; // "/"\n}',
+      expect: `${prompt_multiline}${prompt_multiline}undefined\n${prompt_unix}`
+    },
+
+    {
+      client: client_unix, send: '(function() {\nreturn /foo/ / /bar/;\n}())',
+      expect: `${prompt_multiline}${prompt_multiline}NaN\n${prompt_unix}`
+    },
+
+    {
+      client: client_unix, send: '(function() {\nif (false) {} /bar"/;\n}())',
+      expect: `${prompt_multiline}${prompt_multiline}undefined\n${prompt_unix}`
+    },
+    // Do not parse `...[]` as a REPL keyword
+    { client: client_unix, send: '...[]\n',
+      expect: `${prompt_multiline}` },
+    // bring back the repl to prompt
+    { client: client_unix, send: '.break',
+      expect: `${prompt_unix}` }
   ]);
 }
 
@@ -325,7 +409,7 @@ function tcp_test() {
   });
 
   server_tcp.listen(0, function() {
-    var read_buffer = '';
+    let read_buffer = '';
 
     client_tcp = net.createConnection(this.address().port);
 
@@ -337,20 +421,20 @@ function tcp_test() {
         { client: client_tcp, send: '',
           expect: prompt_tcp },
         { client: client_tcp, send: 'invoke_me(333)',
-          expect: ('\'' + 'invoked 333' + '\'\n' + prompt_tcp) },
+          expect: (`'invoked 333'\n${prompt_tcp}`) },
         { client: client_tcp, send: 'a += 1',
-          expect: ('12346' + '\n' + prompt_tcp) },
+          expect: (`12346\n${prompt_tcp}`) },
         { client: client_tcp,
-          send: 'require(' + JSON.stringify(moduleFilename) + ').number',
-          expect: ('42' + '\n' + prompt_tcp) }
+          send: `require(${JSON.stringify(moduleFilename)}).number`,
+          expect: (`42\n${prompt_tcp}`) }
       ]);
     });
 
     client_tcp.on('data', function(data) {
       read_buffer += data.toString('ascii', 0, data.length);
-      console.error('TCP data: ' + JSON.stringify(read_buffer) +
-                   ', expecting ' + JSON.stringify(client_tcp.expect));
-      if (read_buffer.indexOf(prompt_tcp) !== -1) {
+      console.error(`TCP data: ${JSON.stringify(read_buffer)}, expecting ${
+        JSON.stringify(client_tcp.expect)}`);
+      if (read_buffer.includes(prompt_tcp)) {
         assert.strictEqual(client_tcp.expect, read_buffer);
         console.error('match');
         read_buffer = '';
@@ -394,7 +478,7 @@ function unix_test() {
   });
 
   server_unix.on('listening', function() {
-    var read_buffer = '';
+    let read_buffer = '';
 
     client_unix = net.createConnection(common.PIPE);
 
@@ -406,21 +490,21 @@ function unix_test() {
         { client: client_unix, send: '',
           expect: prompt_unix },
         { client: client_unix, send: 'message',
-          expect: ('\'' + message + '\'\n' + prompt_unix) },
+          expect: (`'${message}'\n${prompt_unix}`) },
         { client: client_unix, send: 'invoke_me(987)',
-          expect: ('\'' + 'invoked 987' + '\'\n' + prompt_unix) },
+          expect: (`'invoked 987'\n${prompt_unix}`) },
         { client: client_unix, send: 'a = 12345',
-          expect: ('12345' + '\n' + prompt_unix) },
+          expect: (`12345\n${prompt_unix}`) },
         { client: client_unix, send: '{a:1}',
-          expect: ('{ a: 1 }' + '\n' + prompt_unix) }
+          expect: (`{ a: 1 }\n${prompt_unix}`) }
       ]);
     });
 
     client_unix.on('data', function(data) {
       read_buffer += data.toString('ascii', 0, data.length);
-      console.error('Unix data: ' + JSON.stringify(read_buffer) +
-                   ', expecting ' + JSON.stringify(client_unix.expect));
-      if (read_buffer.indexOf(prompt_unix) !== -1) {
+      console.error(`Unix data: ${JSON.stringify(read_buffer)}, expecting ${
+        JSON.stringify(client_unix.expect)}`);
+      if (read_buffer.includes(prompt_unix)) {
         assert.strictEqual(client_unix.expect, read_buffer);
         console.error('match');
         read_buffer = '';
