@@ -3,7 +3,7 @@ const common = require('../common');
 const assert = require('assert');
 const spawn = require('child_process').spawn;
 
-const child = spawn(process.argv[0], [common.fixturesDir + '/should_exit.js']);
+const child = spawn(process.argv[0], [`${common.fixturesDir}/should_exit.js`]);
 child.stdout.once('data', function() {
   child.kill('SIGINT');
 });
