@@ -6,9 +6,9 @@ const child_process = require('child_process');
 
 const testScript = path.join(common.fixturesDir, 'catch-stdout-error.js');
 
-const cmd = JSON.stringify(process.execPath) + ' ' +
-            JSON.stringify(testScript) + ' | ' +
-            JSON.stringify(process.execPath) + ' ' +
+const cmd = `${JSON.stringify(process.execPath)} ` +
+            `${JSON.stringify(testScript)} | ` +
+            `${JSON.stringify(process.execPath)} ` +
             '-pe "process.stdin.on(\'data\' , () => process.exit(1))"';
 
 const child = child_process.exec(cmd);
@@ -32,6 +32,6 @@ child.on('close', common.mustCall(function(code) {
     process.exit(1);
   }
 
-  assert.deepEqual(output, outputExpect);
+  assert.deepStrictEqual(output, outputExpect);
   console.log('ok');
 }));
