@@ -1,48 +1,44 @@
 'use strict';
-require('../common');
-var assert = require('assert');
-var stream = require('stream');
-var str = 'asdfasdfasdfasdfasdf';
+const common = require('../common');
+const assert = require('assert');
+const stream = require('stream');
+const str = 'asdfasdfasdfasdfasdf';
 
-var r = new stream.Readable({
+const r = new stream.Readable({
   highWaterMark: 5,
   encoding: 'utf8'
 });
 
-var reads = 0;
-var eofed = false;
-var ended = false;
+let reads = 0;
 
-r._read = function(n) {
+function _read() {
   if (reads === 0) {
     setTimeout(function() {
       r.push(str);
-    });
+    }, 1);
     reads++;
   } else if (reads === 1) {
-    var ret = r.push(str);
-    assert.equal(ret, false);
+    const ret = r.push(str);
+    assert.strictEqual(ret, false);
     reads++;
   } else {
-    assert(!eofed);
-    eofed = true;
     r.push(null);
   }
-};
+}
 
-r.on('end', function() {
-  ended = true;
-});
+r._read = common.mustCall(_read, 3);
+
+r.on('end', common.mustCall());
 
 // push some data in to start.
 // we've never gotten any read event at this point.
-var ret = r.push(str);
+const ret = r.push(str);
 // should be false.  > hwm
 assert(!ret);
-var chunk = r.read();
-assert.equal(chunk, str);
+let chunk = r.read();
+assert.strictEqual(chunk, str);
 chunk = r.read();
-assert.equal(chunk, null);
+assert.strictEqual(chunk, null);
 
 r.once('readable', function() {
   // this time, we'll get *all* the remaining data, because
@@ -50,15 +46,8 @@ r.once('readable', function() {
   // us below the hwm, and so it triggered a _read() again,
   // which synchronously added more, which we then return.
   chunk = r.read();
-  assert.equal(chunk, str + str);
+  assert.strictEqual(chunk, str + str);
 
   chunk = r.read();
-  assert.equal(chunk, null);
-});
-
-process.on('exit', function() {
-  assert(eofed);
-  assert(ended);
-  assert.equal(reads, 2);
-  console.log('ok');
+  assert.strictEqual(chunk, null);
 });
