@@ -1,15 +1,14 @@
 'use strict';
 require('../common');
-var assert = require('assert');
+const assert = require('assert');
 
-// This test verifies that stream.unshift(Buffer(0)) or
+// This test verifies that stream.unshift(Buffer.alloc(0)) or
 // stream.unshift('') does not set state.reading=false.
-var Readable = require('stream').Readable;
+const Readable = require('stream').Readable;
 
-var r = new Readable();
-var nChunks = 10;
-var chunk = new Buffer(10);
-chunk.fill('x');
+const r = new Readable();
+let nChunks = 10;
+const chunk = Buffer.alloc(10, 'x');
 
 r._read = function(n) {
   setImmediate(function() {
@@ -17,10 +16,10 @@ r._read = function(n) {
   });
 };
 
-var readAll = false;
-var seen = [];
+let readAll = false;
+const seen = [];
 r.on('readable', function() {
-  var chunk;
+  let chunk;
   while (chunk = r.read()) {
     seen.push(chunk.toString());
     // simulate only reading a certain amount of the data,
@@ -28,14 +27,13 @@ r.on('readable', function() {
     // stream, like a parser might do.  We just fill it with
     // 'y' so that it's easy to see which bits were touched,
     // and which were not.
-    var putBack = new Buffer(readAll ? 0 : 5);
-    putBack.fill('y');
+    const putBack = Buffer.alloc(readAll ? 0 : 5, 'y');
     readAll = !readAll;
     r.unshift(putBack);
   }
 });
 
-var expect =
+const expect =
   [ 'xxxxxxxxxx',
     'yyyyy',
     'xxxxxxxxxx',
@@ -56,6 +54,6 @@ var expect =
     'yyyyy' ];
 
 r.on('end', function() {
-  assert.deepEqual(seen, expect);
+  assert.deepStrictEqual(seen, expect);
   console.log('ok');
 });
