@@ -1,20 +1,20 @@
 'use strict';
 const common = require('../common');
-var R = require('_stream_readable');
-var W = require('_stream_writable');
-var assert = require('assert');
+const R = require('_stream_readable');
+const W = require('_stream_writable');
+const assert = require('assert');
 
-var src = new R({encoding: 'base64'});
-var dst = new W();
-var hasRead = false;
-var accum = [];
-var timeout;
+const src = new R({encoding: 'base64'});
+const dst = new W();
+let hasRead = false;
+const accum = [];
+let timeout;
 
 src._read = function(n) {
   if (!hasRead) {
     hasRead = true;
     process.nextTick(function() {
-      src.push(new Buffer('1'));
+      src.push(Buffer.from('1'));
       src.push(null);
     });
   }
@@ -26,7 +26,7 @@ dst._write = function(chunk, enc, cb) {
 };
 
 src.on('end', function() {
-  assert.equal(Buffer.concat(accum) + '', 'MQ==');
+  assert.strictEqual(String(Buffer.concat(accum)), 'MQ==');
   clearTimeout(timeout);
 });
 
