@@ -1,13 +1,13 @@
 'use strict';
 require('../common');
 
-var util = require('util');
-var stream = require('stream');
+const util = require('util');
+const stream = require('stream');
 
 
-var Read = function() {
+function Read() {
   stream.Readable.call(this);
-};
+}
 util.inherits(Read, stream.Readable);
 
 Read.prototype._read = function(size) {
@@ -16,9 +16,9 @@ Read.prototype._read = function(size) {
 };
 
 
-var Write = function() {
+function Write() {
   stream.Writable.call(this);
-};
+}
 util.inherits(Write, stream.Writable);
 
 Write.prototype._write = function(buffer, encoding, cb) {
@@ -26,10 +26,10 @@ Write.prototype._write = function(buffer, encoding, cb) {
   this.emit('alldone');
 };
 
-var read = new Read();
-var write = new Write();
+const read = new Read();
+const write = new Write();
 
-write.once('error', function(err) {});
+write.once('error', () => {});
 write.once('alldone', function(err) {
   console.log('ok');
 });
@@ -39,4 +39,3 @@ process.on('exit', function(c) {
 });
 
 read.pipe(write);
-
