@@ -3,42 +3,37 @@
 // Cache session and close connection.  Use session on second connection.
 // ASSERT resumption.
 
-var common = require('../common');
-var assert = require('assert');
-
-if (!common.hasCrypto) {
+const common = require('../common');
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
-var tls = require('tls');
 
-var fs = require('fs');
+const assert = require('assert');
+const tls = require('tls');
+const fs = require('fs');
 
-var options = {
-  key: fs.readFileSync(common.fixturesDir + '/keys/agent2-key.pem'),
-  cert: fs.readFileSync(common.fixturesDir + '/keys/agent2-cert.pem')
+const options = {
+  key: fs.readFileSync(`${common.fixturesDir}/keys/agent2-key.pem`),
+  cert: fs.readFileSync(`${common.fixturesDir}/keys/agent2-cert.pem`)
 };
 
-var big = new Buffer(2 * 1024 * 1024);
-
-big.fill('Y');
+const big = Buffer.alloc(2 * 1024 * 1024, 'Y');
 
 // create server
-var server = tls.createServer(options, common.mustCall(function(socket) {
+const server = tls.createServer(options, common.mustCall(function(socket) {
   socket.end(big);
   socket.destroySoon();
 }));
 
 // start listening
 server.listen(0, common.mustCall(function() {
-  var client = tls.connect({
+  const client = tls.connect({
     port: this.address().port,
     rejectUnauthorized: false
   }, common.mustCall(function() {
-    var bytesRead = 0;
+    let bytesRead = 0;
 
     client.on('readable', function() {
-      var d = client.read();
+      const d = client.read();
       if (d)
         bytesRead += d.length;
     });
