@@ -3,10 +3,8 @@
 'use strict';
 const common = require('../common');
 
-if (!common.hasCrypto) {
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
 
 const assert = require('assert');
 const tls = require('tls');
@@ -19,24 +17,22 @@ if (process.env.CHILD) {
 
 const env = {
   CHILD: 'yes',
-  NODE_EXTRA_CA_CERTS: common.fixturesDir + '/no-such-file-exists',
+  NODE_EXTRA_CA_CERTS: `${common.fixturesDir}/no-such-file-exists`,
 };
 
-var opts = {
+const opts = {
   env: env,
   silent: true,
 };
-var stderr = '';
+let stderr = '';
 
 fork(__filename, opts)
   .on('exit', common.mustCall(function(status) {
-    assert.equal(status, 0, 'client did not succeed in connecting');
+    assert.strictEqual(status, 0, 'client did not succeed in connecting');
   }))
   .on('close', common.mustCall(function() {
-    assert(stderr.match(new RegExp(
-      'Warning: Ignoring extra certs from.*no-such-file-exists' +
-      '.* load failed:.*No such file or directory'
-    )), stderr);
+    const re = /Warning: Ignoring extra certs from.*no-such-file-exists.* load failed:.*No such file or directory/;
+    assert(re.test(stderr), stderr);
   }))
   .stderr.setEncoding('utf8').on('data', function(str) {
     stderr += str;
