@@ -2,10 +2,8 @@
 
 const common = require('../common');
 
-if (!common.hasCrypto) {
+if (!common.hasCrypto)
   common.skip('node compiled without crypto.');
-  return;
-}
 
 const assert = require('assert');
 const tls = require('tls');
@@ -13,7 +11,7 @@ const fs = require('fs');
 const path = require('path');
 
 const pfx = fs.readFileSync(
-    path.join(common.fixturesDir, 'keys', 'agent1-pfx.pem'));
+  path.join(common.fixturesDir, 'keys', 'agent1-pfx.pem'));
 
 const server = tls.createServer({
   pfx: pfx,
@@ -21,10 +19,14 @@ const server = tls.createServer({
   requestCert: true,
   rejectUnauthorized: false
 }, common.mustCall(function(c) {
-  assert(c.authorizationError === null, 'authorizationError must be null');
+  assert.strictEqual(
+    c.authorizationError,
+    null,
+    'authorizationError must be null'
+  );
   c.end();
 })).listen(0, function() {
-  var client = tls.connect({
+  const client = tls.connect({
     port: this.address().port,
     pfx: pfx,
     passphrase: 'sample',
