@@ -1,11 +1,11 @@
 'use strict';
 require('../common');
-var assert = require('assert');
-var vm = require('vm');
+const assert = require('assert');
+const vm = require('vm');
 
-var sandbox = { x: 3 };
+const sandbox = { x: 3 };
 
-var ctx = vm.createContext(sandbox);
+const ctx = vm.createContext(sandbox);
 
 assert.strictEqual(vm.runInContext('x;', ctx), 3);
 vm.runInContext('y = 4;', ctx);
