@@ -1,13 +1,13 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var path = require('path');
-var fs = require('fs');
+const common = require('../common');
+const assert = require('assert');
+const path = require('path');
+const fs = require('fs');
 
-var FILENAME = path.join(common.tmpDir, 'watch-me');
-var TIMEOUT = 1300;
+const FILENAME = path.join(common.tmpDir, 'watch-me');
+const TIMEOUT = 1300;
 
-var nevents = 0;
+let nevents = 0;
 
 try {
   fs.unlinkSync(FILENAME);
@@ -19,14 +19,14 @@ fs.watchFile(FILENAME, {interval: TIMEOUT - 250}, function(curr, prev) {
   console.log([curr, prev]);
   switch (++nevents) {
     case 1:
-      assert.equal(common.fileExists(FILENAME), false);
+      assert.strictEqual(common.fileExists(FILENAME), false);
       break;
     case 2:
     case 3:
-      assert.equal(common.fileExists(FILENAME), true);
+      assert.strictEqual(common.fileExists(FILENAME), true);
       break;
     case 4:
-      assert.equal(common.fileExists(FILENAME), false);
+      assert.strictEqual(common.fileExists(FILENAME), false);
       fs.unwatchFile(FILENAME);
       break;
     default:
@@ -35,7 +35,7 @@ fs.watchFile(FILENAME, {interval: TIMEOUT - 250}, function(curr, prev) {
 });
 
 process.on('exit', function() {
-  assert.equal(nevents, 4);
+  assert.strictEqual(nevents, 4);
 });
 
 setTimeout(createFile, TIMEOUT);
