@@ -1,8 +1,10 @@
 'use strict';
 // Flags: --expose_gc
 
+const common = require('../common');
+
 function newBuffer(size, value) {
-  var buffer = new Buffer(size);
+  const buffer = Buffer.allocUnsafe(size);
   while (size--) {
     buffer[size] = value;
   }
@@ -11,35 +13,33 @@ function newBuffer(size, value) {
   return buffer;
 }
 
-
-var common = require('../common');
-var fs = require('fs');
-var testFileName = require('path').join(common.tmpDir, 'GH-814_testFile.txt');
-var testFileFD = fs.openSync(testFileName, 'w');
+const fs = require('fs');
+const testFileName = require('path').join(common.tmpDir, 'GH-814_testFile.txt');
+const testFileFD = fs.openSync(testFileName, 'w');
 console.log(testFileName);
 
 
-var kBufSize = 128 * 1024;
-var PASS = true;
-var neverWrittenBuffer = newBuffer(kBufSize, 0x2e); //0x2e === '.'
-var bufPool = [];
+const kBufSize = 128 * 1024;
+let PASS = true;
+const neverWrittenBuffer = newBuffer(kBufSize, 0x2e); //0x2e === '.'
+const bufPool = [];
 
 
-var tail = require('child_process').spawn('tail', ['-f', testFileName]);
+const tail = require('child_process').spawn('tail', ['-f', testFileName]);
 tail.stdout.on('data', tailCB);
 
 function tailCB(data) {
-  PASS = data.toString().indexOf('.') < 0;
+  PASS = !data.toString().includes('.');
 }
 
 
-var timeToQuit = Date.now() + 8e3; //Test during no more than this seconds.
+const timeToQuit = Date.now() + 8e3; //Test during no more than this seconds.
 (function main() {
 
   if (PASS) {
     fs.write(testFileFD, newBuffer(kBufSize, 0x61), 0, kBufSize, -1, cb);
-    gc();
-    var nuBuf = new Buffer(kBufSize);
+    global.gc();
+    const nuBuf = Buffer.allocUnsafe(kBufSize);
     neverWrittenBuffer.copy(nuBuf);
     if (bufPool.push(nuBuf) > 100) {
       bufPool.length = 0;
