@@ -1,32 +1,32 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
+const common = require('../common');
+const assert = require('assert');
 
-var execSync = require('child_process').execSync;
-var execFileSync = require('child_process').execFileSync;
+const { execFileSync, execSync } = require('child_process');
 
-var TIMER = 200;
-var SLEEP = 2000;
+const TIMER = 200;
+const SLEEP = 2000;
 
-var start = Date.now();
-var err;
-var caught = false;
+const start = Date.now();
+let err;
+let caught = false;
 
 // Verify that stderr is not accessed when a bad shell is used
 assert.throws(
   function() { execSync('exit -1', {shell: 'bad_shell'}); },
-  /ENOENT/,
+  /spawnSync bad_shell ENOENT/,
   'execSync did not throw the expected exception!'
 );
 assert.throws(
   function() { execFileSync('exit -1', {shell: 'bad_shell'}); },
-  /ENOENT/,
+  /spawnSync bad_shell ENOENT/,
   'execFileSync did not throw the expected exception!'
 );
 
+let cmd, ret;
 try {
-  var cmd = `"${process.execPath}" -e "setTimeout(function(){}, ${SLEEP});"`;
-  var ret = execSync(cmd, {timeout: TIMER});
+  cmd = `"${process.execPath}" -e "setTimeout(function(){}, ${SLEEP});"`;
+  ret = execSync(cmd, {timeout: TIMER});
 } catch (e) {
   caught = true;
   assert.strictEqual(e.errno, 'ETIMEDOUT');
@@ -34,7 +34,7 @@ try {
 } finally {
   assert.strictEqual(ret, undefined, 'we should not have a return value');
   assert.strictEqual(caught, true, 'execSync should throw');
-  var end = Date.now() - start;
+  const end = Date.now() - start;
   assert(end < SLEEP);
   assert(err.status > 128 || err.signal);
 }
@@ -43,33 +43,33 @@ assert.throws(function() {
   execSync('iamabadcommand');
 }, /Command failed: iamabadcommand/);
 
-var msg = 'foobar';
-var msgBuf = new Buffer(msg + '\n');
+const msg = 'foobar';
+const msgBuf = Buffer.from(`${msg}\n`);
 
 // console.log ends every line with just '\n', even on Windows.
 
-cmd = `"${process.execPath}" -e "console.log(\'${msg}\');"`;
+cmd = `"${process.execPath}" -e "console.log('${msg}');"`;
 
 ret = execSync(cmd);
 
 assert.strictEqual(ret.length, msgBuf.length);
-assert.deepEqual(ret, msgBuf, 'execSync result buffer should match');
+assert.deepStrictEqual(ret, msgBuf, 'execSync result buffer should match');
 
 ret = execSync(cmd, { encoding: 'utf8' });
 
-assert.strictEqual(ret, msg + '\n', 'execSync encoding result should match');
+assert.strictEqual(ret, `${msg}\n`, 'execSync encoding result should match');
 
-var args = [
+const args = [
   '-e',
   `console.log("${msg}");`
 ];
 ret = execFileSync(process.execPath, args);
 
-assert.deepEqual(ret, msgBuf);
+assert.deepStrictEqual(ret, msgBuf);
 
 ret = execFileSync(process.execPath, args, { encoding: 'utf8' });
 
-assert.strictEqual(ret, msg + '\n',
+assert.strictEqual(ret, `${msg}\n`,
                    'execFileSync encoding result should match');
 
 // Verify that the cwd option works - GH #7824
@@ -82,11 +82,11 @@ assert.strictEqual(ret, msg + '\n',
 }
 
 // Verify that stderr is not accessed when stdio = 'ignore' - GH #7966
-(function() {
+{
   assert.throws(function() {
     execSync('exit -1', {stdio: 'ignore'});
   }, /Command failed: exit -1/);
-})();
+}
 
 // Verify the execFileSync() behavior when the child exits with a non-zero code.
 {
