@@ -1,5 +1,5 @@
 'use strict';
-var common = require('../common');
+const common = require('../common');
 
 // this test only fails with CentOS 6.3 using kernel version 2.6.32
 // On other linuxes and darwin, the `read` call gets an ECONNRESET in
@@ -18,9 +18,8 @@ switch (process.argv[2]) {
 }
 
 function server() {
-  var net = require('net');
-  var content = new Buffer(64 * 1024 * 1024);
-  content.fill('#');
+  const net = require('net');
+  const content = Buffer.alloc(64 * 1024 * 1024, '#');
   net.createServer(function(socket) {
     this.close();
     socket.on('end', function() {
@@ -36,8 +35,8 @@ function server() {
 }
 
 function client() {
-  var net = require('net');
-  var client = net.connect({
+  const net = require('net');
+  const client = net.connect({
     host: common.localhostIPv4,
     port: common.PORT
   }, function() {
@@ -46,23 +45,15 @@ function client() {
 }
 
 function parent() {
-  var spawn = require('child_process').spawn;
-  var node = process.execPath;
+  const spawn = require('child_process').spawn;
+  const node = process.execPath;
 
-  setTimeout(function() {
-    if (s) s.kill();
-    if (c) c.kill();
-    setTimeout(function() {
-      throw new Error('hang');
-    });
-  }, common.platformTimeout(2000)).unref();
-
-  var s = spawn(node, [__filename, 'server'], {
-    env: Object.assign(process.env, {
+  const s = spawn(node, [__filename, 'server'], {
+    env: Object.assign({}, process.env, {
       NODE_DEBUG: 'net'
     })
   });
-  var c;
+  let c;
 
   wrap(s.stderr, process.stderr, 'SERVER 2>');
   wrap(s.stdout, process.stdout, 'SERVER 1>');
@@ -80,8 +71,7 @@ function parent() {
     inp.on('data', function(c) {
       c = c.trim();
       if (!c) return;
-      out.write(w + c.split('\n').join('\n' + w) + '\n');
+      out.write(`${w}${c.split('\n').join(`\n${w}`)}\n`);
     });
   }
 }
-
