@@ -36,7 +36,7 @@
 echo "Searching $1 to write out expfile to $2"
 
 # this special sequence must be at the start of the exp file
-echo "#!." > $2
+echo "#!." > $2.tmp
 
 # pull the symbols from the .a files
 find $1 -name "*.a" | grep -v gtest \
@@ -45,4 +45,6 @@ find $1 -name "*.a" | grep -v gtest \
       if ((($2 == "T") || ($2 == "D") || ($2 == "B")) &&
           (substr($3,1,1) != ".")) { print $3 }
     }' \
-  | sort -u >> $2
+  | sort -u >> $2.tmp
+
+mv -f $2.tmp $2
